@@ -1,7 +1,7 @@
 /*
  * AIDE (Advanced Intrusion Detection Environment)
  *
- * Copyright (C) 1999-2002, 2019-2021 Rami Lehti, Pablo Virolainen,
+ * Copyright (C) 1999-2002, 2019-2021,2023 Rami Lehti, Pablo Virolainen,
  *               Hannes von Haugwitz
  *
  * This program is free software; you can redistribute it and/or
@@ -24,6 +24,6 @@
 #include <stdbool.h>
 #include "url.h"
 
-void* be_init(bool ,url_t*, bool, bool, int, char*, char*);
+void* be_init(url_t*, bool, bool, bool, int, char*, char*, bool*);
 
 #endif /* _BE_H_INCLUDED */
