@@ -1,7 +1,7 @@
 /*
  * AIDE (Advanced Intrusion Detection Environment)
  *
- * Copyright (C) 2019-2022 Hannes von Haugwitz
+ * Copyright (C) 2019-2025 Hannes von Haugwitz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -48,7 +48,6 @@ typedef enum config_option {
     REPORT_URL_OPTION,
     ROOT_PREFIX_OPTION,
     WARN_DEAD_SYMLINKS_OPTION,
-    VERBOSE_OPTION,
     CONFIG_VERSION,
     CONFIG_CHECK_WARN_UNRESTRICTED_RULES,
     REPORT_FORMAT_OPTION,
@@ -101,14 +100,21 @@ typedef enum bool_operator {
         BOOL_OP_DEFINED,
         BOOL_OP_HOSTNAME,
         BOOL_OP_EXISTS,
+        BOOL_OP_VERSION_GE,
 } bool_operator;
 
 typedef struct bool_expression {
     bool_operator op;
 
-    string_expression* expr;
-    struct bool_expression* left;
-    struct bool_expression* right;
+    union {
+        string_expression* _str;
+        struct bool_expression* _bool;
+    } left;
+    union {
+        string_expression* _str;
+        struct bool_expression* _bool;
+    } right;
+
 } bool_expression;
 
 typedef struct if_condition {
@@ -152,9 +158,14 @@ typedef struct group_statement {
     attribute_expression *expr;
 } group_statement;
 
-typedef struct restriction_expression {
+typedef struct ft_restriction_expression {
     char* right;
-    struct restriction_expression* left;
+    struct ft_restriction_expression* left;
+} ft_restriction_expression;
+
+typedef struct restriction_expression {
+    struct ft_restriction_expression* f_types;
+    char* fs_type;
 } restriction_expression;
 
 typedef struct rule_statement {
@@ -220,11 +231,13 @@ ast* new_rule_statement(AIDE_RULE_TYPE, string_expression*, restriction_expressi
 
 if_condition* new_if_condition(struct bool_expression*);
 
-bool_expression* new_string_bool_expression(bool_operator, string_expression*);
+bool_expression* new_string_bool_expression(bool_operator, string_expression*, string_expression*);
 bool_expression* new_bool_expression(bool_operator, bool_expression*, bool_expression*);
 
 attribute_expression* new_attribute_expression(attribute_operator, attribute_expression*, char*);
-restriction_expression* new_restriction_expression(restriction_expression*, char*);
+
+ft_restriction_expression* new_ft_restriction_expression(ft_restriction_expression*, char*);
+restriction_expression* new_restriction_expression(ft_restriction_expression*, char*);
 
 void deep_free(ast*);
 
