@@ -1,7 +1,7 @@
 /*
  * AIDE (Advanced Intrusion Detection Environment)
  *
- * Copyright (C) 1999-2002, 2004-2005, 2013, 2016, 2020, 2022, 2023 Rami Lehti,
+ * Copyright (C) 1999-2002, 2004-2005, 2013, 2016, 2020, 2022-2025 Rami Lehti,
  *               Pablo Virolainen, Richard van den Berg, Hannes von Haugwitz
  *
  * This program is free software; you can redistribute it and/or
@@ -24,7 +24,6 @@
 
 #include <stdbool.h>
 #include <stdio.h>
-#include <time.h>
 #include "db_config.h"
 #include "util.h"
 
@@ -34,18 +33,24 @@ typedef enum {
     DB_TYPE_NEW,
 } DB_TYPE;
 
+typedef struct {
+    db_line* line;
+    bool limit;
+} db_entry_t;
+
 byte* base64tobyte(char*, int, size_t *);
-time_t base64totime_t(char*, database*, const char*);
+
+db_line* db_char2line(char**, database*);
 
 int db_init(database*, bool, bool);
 
-db_line* db_readline(database*);
+db_entry_t db_readline(database*, bool);
 
 int db_writespec(db_config*);
 
 int db_writeline(db_line*,db_config*);
 
-void db_close();
+void db_close(void);
 
 void free_db_line(db_line* dl);
 
@@ -61,6 +66,10 @@ void free_db_line(db_line* dl);
 #define NODE_MOVED_IN     (1<<12)
 #define NODE_ALLOW_NEW    (1<<13)
 #define NODE_ALLOW_RM	  (1<<14)
-#define NODE_CHECK_INODE_CHILDS (1<<15)
+#define NODE_CHECK_INODE           (1<<15)
+#define NODE_HAS_SUB_RULES         (1<<16)
+
+#define LOG_DB_FORMAT_LINE(log_level, format, ...) \
+    log_msg(log_level, "%s:%li: " format , (db->url)->raw, db->lineno, __VA_ARGS__);
 
 #endif
