@@ -1,7 +1,7 @@
 /*
  * AIDE (Advanced Intrusion Detection Environment)
  *
- * Copyright (C) 1999-2002, 2004-2006, 2010-2013, 2015-2016, 2019-2022
+ * Copyright (C) 1999-2002, 2004-2006, 2010-2013, 2015-2016, 2019-2024
  *               Rami Lehti, Pablo Virolainen, Richard van den Berg,
  *               Hannes von Haugwitz
  *
@@ -27,18 +27,16 @@
 #define PCRE2_CODE_UNIT_WIDTH 8
 #include <pcre2.h>
 #include <stdbool.h>
-#include <stdio.h>
 #include <sys/types.h>
 #ifdef WITH_ZLIB
 #include <zlib.h>
 #endif
 #include "attributes.h"
-#include "hashsum.h"
 #include "db_line.h"
 #include "list.h"
 #include "report.h"
+#include "file.h"
 #include "rx_rule.h"
-#include "util.h"
 #include "url.h"
 
 
@@ -51,6 +49,7 @@
 #define DO_COMPARE  (1<<1)
 #define DO_DIFF     (1<<2)
 #define DO_DRY_RUN  (1<<3)
+#define DO_LIST     (1<<4)
 
 /* TIMEBUFSIZE should be exactly ceil(sizeof(time_t)*8*ln(2)/ln(10))
  * Now it is ceil(sizeof(time_t)*2.5)
@@ -78,6 +77,12 @@
 /*    void* local; */  
 /*  }_db_config ; */
 
+typedef enum {
+   DB_FLAG_NONE     =0,
+   DB_FLAG_CREATED  =1,
+   DB_FLAG_PARSE    =2,
+} DB_FLAG;
+
 typedef struct database {
     url_t* url;
 
@@ -93,10 +98,11 @@ typedef struct database {
     long lineno;
     ATTRIBUTE* fields;
     int num_fields;
-    void *buffer_state;
     struct md_container *mdc;
     struct db_line *db_line;
 
+    DB_FLAG flags;
+
 } database;
 
 typedef struct db_config {
@@ -116,11 +122,11 @@ typedef struct db_config {
 
   DB_ATTR_TYPE db_out_attrs;
 
-  char *check_path;
-  RESTRICTION_TYPE check_file_type;
+  file_t check_file;
   
   char* config_file;
   char* config_version;
+  char* aide_version;
   bool config_check_warn_unrestricted_rules;
 
   int database_add_metadata;
@@ -150,9 +156,10 @@ typedef struct db_config {
   /* What are we supposed to do */
   int action;
 
-#ifdef WITH_PTHREAD
   long num_workers;
-#endif
+
+  int progress;
+  bool no_color;
 
   time_t start_time;
   time_t end_time;
@@ -178,6 +185,8 @@ typedef struct db_config {
 
   struct seltree* tree;
 
+  int print_details_width;
+
 } db_config;
 
 #endif
