@@ -1,7 +1,7 @@
 /*
  * AIDE (Advanced Intrusion Detection Environment)
  *
- * Copyright (C) 1999-2002, 2006, 2020, 2022 Rami Lehti, Pablo Virolainen,
+ * Copyright (C) 1999-2002, 2006, 2020, 2022, 2024, 2025 Rami Lehti, Pablo Virolainen,
  *               Richard van den Berg, Hannes von Haugwitz
  *
  * This program is free software; you can redistribute it and/or
@@ -22,19 +22,15 @@
 #ifndef _DB_FILE_H_INCLUDED
 #define _DB_FILE_H_INCLUDED
 
-#include "config.h"
-#ifdef WITH_ZLIB
-#include <zlib.h>
-#endif
+#include "db.h"
 #include "db_config.h"
-#include "url.h"
+#include <stdbool.h>
+
+db_entry_t db_readline_file(database*, bool);
 
-char** db_readline_file(database*);
 int db_writespec_file(db_config*);
-int db_writeline_file(db_line* line,db_config* conf,url_t* url);
-int db_close_file(db_config* conf);
-#ifdef WITH_ZLIB
-void handle_gzipped_input(int out,gzFile*);
-#endif
+int db_writeline_file(db_line*);
+
+int db_close_file(db_config*);
 
 #endif
