@@ -1,7 +1,7 @@
 /*
  * AIDE (Advanced Intrusion Detection Environment)
  *
- * Copyright (C) 2020, 2022, 2023 Hannes von Haugwitz
+ * Copyright (C) 2020, 2022, 2023, 2025 Hannes von Haugwitz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -21,7 +21,6 @@
 #ifndef _LOG_H_INCLUDED
 #define  _LOG_H_INCLUDED
 
-#include "config.h"
 #include <stdbool.h>
 
 /* log levels */
@@ -31,21 +30,19 @@ typedef enum { /* preserve order */
     LOG_LEVEL_WARNING = 2,
     LOG_LEVEL_NOTICE = 3,
     LOG_LEVEL_INFO = 4,
-    LOG_LEVEL_RULE = 5,
-    LOG_LEVEL_COMPARE = 6,
+    LOG_LEVEL_COMPARE = 5,
+    LOG_LEVEL_RULE = 6,
     LOG_LEVEL_CONFIG = 7,
     LOG_LEVEL_DEBUG = 8,
-    LOG_LEVEL_THREAD = 9,
-    LOG_LEVEL_TRACE = 10,
+    LOG_LEVEL_LIMIT = 9,
+    LOG_LEVEL_THREAD = 10,
+    LOG_LEVEL_TRACE = 11,
 } LOG_LEVEL;
 
-#ifdef WITH_PTHREAD
-void log_init();
-#endif
-
-bool is_log_level_unset();
+bool is_log_level_unset(void);
 
 void set_log_level(LOG_LEVEL);
+void set_colored_log(bool);
 
 const char * get_log_level_name(LOG_LEVEL);
 
@@ -53,9 +50,11 @@ LOG_LEVEL get_log_level_from_string(char*);
 
 LOG_LEVEL toogle_log_level(LOG_LEVEL);
 
-void log_msg(LOG_LEVEL, const char* ,...);
-
-void stderr_msg(const char* ,...);
+void log_msg(LOG_LEVEL, const char* ,...)
+#ifdef __GNUC__
+    __attribute__ ((format (printf, 2, 3)))
+#endif
+;
 
 #define LOG_CONFIG_FORMAT_LINE(log_level, format, ...) \
     if (linebuf) { \
@@ -71,4 +70,6 @@ void stderr_msg(const char* ,...);
         log_msg(log_level,"%s:%d: " format " (line: '%s')", filename, linenumber, __VA_ARGS__, linebuf); \
     }
 
+#define LOG_WHOAMI(log_level, format, ...) log_msg(log_level, "%*s%s" format, whoami?10:0, whoami?whoami:"", whoami?": ":"", __VA_ARGS__);
+
 #endif
