@@ -1,7 +1,7 @@
 /*
  * AIDE (Advanced Intrusion Detection Environment)
  *
- * Copyright (C) 1999-2002, 2010, 2020-2022 Rami Lehti, Pablo Virolainen,
+ * Copyright (C) 1999-2002, 2010, 2020-2024 Rami Lehti, Pablo Virolainen,
  *               Hannes von Haugwitz
  *
  * This program is free software; you can redistribute it and/or
@@ -22,22 +22,20 @@
 #ifndef _SELTREE_H_INCLUDED
 #define _SELTREE_H_INCLUDED
 #include "log.h"
+#include "file.h"
 #include "rx_rule.h"
-#include "seltree_struct.h"
 
-seltree* init_tree();
+typedef struct seltree seltree;
 
-seltree* new_seltree_node(seltree*, char*, int, rx_rule*);
+seltree* init_tree(void);
 
 seltree* get_seltree_node(seltree* ,char*);
+seltree* get_or_create_seltree_node(seltree*, char *);
 
-rx_rule * add_rx_to_tree(char *, RESTRICTION_TYPE, int, seltree *, int, char *, char *);
+rx_rule * add_rx_to_tree(char *, rx_restriction_t, AIDE_RULE_TYPE, seltree *, int, char *, char *, char **);
 
-int check_seltree(seltree *, char *, RESTRICTION_TYPE, rx_rule* *);
-
-int treedepth(seltree *);
+match_t check_seltree(seltree *, file_t, bool, const char *);
 
 void log_tree(LOG_LEVEL, seltree *, int);
-
-char* strgetndirname(char* ,int);
+bool is_tree_empty(seltree *);
 #endif /* _SELTREE_H_INCLUDED*/
