@@ -1,7 +1,7 @@
 /*
  * AIDE (Advanced Intrusion Detection Environment)
  *
- * Copyright (C) 1999-2006, 2010-2013, 2015-2017, 2019-2023 Rami Lehti,
+ * Copyright (C) 1999-2006, 2010-2013, 2015-2017, 2019-2025 Rami Lehti,
  *               Pablo Virolainen, Mike Markley, Richard van den Berg,
  *               Hannes von Haugwitz
  *
@@ -23,6 +23,7 @@
 #include "config.h"
 
 #include <sys/stat.h>
+#include <sys/ioctl.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -31,14 +32,14 @@
 #include <time.h>
 #include <stdbool.h>
 #include <string.h>
+#define PCRE2_CODE_UNIT_WIDTH 8
+#include <pcre2.h>
 
-#if HAVE_UNISTD_H
 #include <unistd.h>
-#endif
 
 #include "attributes.h"
 #include "hashsum.h"
-#include "rx_rule.h"
+#include "file.h"
 #include "url.h"
 #include "commandconf.h"
 #include "report.h"
@@ -46,6 +47,7 @@
 #include "db_disk.h"
 #include "db.h"
 #include "log.h"
+#include "progress.h"
 #include "seltree.h"
 #include "errorcodes.h"
 #include "gen_list.h"
@@ -62,11 +64,6 @@ char* after = NULL;
 #define MAXHOSTNAMELEN 256
 #endif
 
-#ifdef WITH_GCRYPT
-#include <gcrypt.h>
-#define NEED_LIBGCRYPT_VERSION "1.8.0"
-#endif
-
 static void usage(int exitvalue)
 {
   fprintf(stdout,
@@ -77,8 +74,9 @@ static void usage(int exitvalue)
 	    "  -n, --dry-init\tTraverse the file system and match each file against rule tree\n"
 	    "  -C, --check\t\tCheck the database\n"
 	    "  -u, --update\t\tCheck and update the database non-interactively\n"
-	    "  -E, --compare\t\tCompare two databases\n\n"
-	    "Miscellaneous:\n"
+	    "  -E, --compare\t\tCompare two databases\n"
+	    "      --list\t\tList the entries of the database in human readable format\n"
+	    "\nMiscellaneous:\n"
 	    "  -D,\t\t\t--config-check\t\t\tTest the configuration file\n"
 	    "  -p FILE_TYPE:PATH\t--path-check=FILE_TYPE:PATH\tMatch file type and path against rule tree\n"
 	    "  -v,\t\t\t--version\t\t\tShow version of AIDE and compilation options\n"
@@ -89,8 +87,10 @@ static void usage(int exitvalue)
 	    "  -B \"OPTION\"\t--before=\"OPTION\"\tBefore configuration file is read define OPTION\n"
 	    "  -A \"OPTION\"\t--after=\"OPTION\"\tAfter configuration file is read define OPTION\n"
 	    "  -L LEVEL\t--log-level=LEVEL\tSet log message level to LEVEL\n"
-	    "  -W WORKERS\t--workers=WORKERS\tNumber of simultaneous workers (threads) for file attribute processing (i.a. hashsum calculation)"
-	    "\n"), AIDEVERSION
+	    "  -W WORKERS\t--workers=WORKERS\tNumber of simultaneous workers (threads) for file attribute processing (i.a. hashsum calculation)\n"
+	    "  \t\t--no-progress\t\tTurn progress off explicitly\n"
+	    "  \t\t--no-color\t\tTUrn color off explicitly\n"
+	    ), conf->aide_version
 	  );
   
   exit(exitvalue);
@@ -98,45 +98,54 @@ static void usage(int exitvalue)
 
 static void sig_handler(int);
 
-static void init_sighandler()
+static void init_db_sighandler(void)
 {
-  log_msg(LOG_LEVEL_DEBUG, "initialize signal handler for SIGTERM, SIGUSR1 and SIGHUP");
+  log_msg(LOG_LEVEL_DEBUG, "initialize signal handler for SIGINT");
+  signal(SIGINT,sig_handler);
+  log_msg(LOG_LEVEL_DEBUG, "initialize signal handler for SIGTERM");
   signal(SIGTERM,sig_handler);
-  signal(SIGUSR1,sig_handler);
+  log_msg(LOG_LEVEL_DEBUG, "initialize signal handler for SIGHUP");
   signal(SIGHUP,sig_handler);
-
-  return;
-}
-
-static void init_crypto_lib() {
-/* libmhash does not need to be initialized */
-#ifdef WITH_GCRYPT
-  if(!gcry_check_version(NEED_LIBGCRYPT_VERSION)) {
-      log_msg(LOG_LEVEL_ERROR, "libgcrypt is too old (need %s, have %s)", NEED_LIBGCRYPT_VERSION, gcry_check_version (NULL));
-      exit(VERSION_MISMATCH_ERROR);
-  }
-  gcry_control(GCRYCTL_DISABLE_SECMEM, 0);
-  gcry_control(GCRYCTL_INITIALIZATION_FINISHED, 0);
-#endif
 }
 
 static void sig_handler(int signum)
 {
+    struct winsize winsize;
     char *str;
     switch(signum){
-        case SIGHUP :
-          str = "Caught SIGHUP. Ignoring\n";
-          (void) !write(STDERR_FILENO ,str, strlen(str));
-          break;
+        case SIGINT :
         case SIGTERM :
-           str = "Caught SIGTERM. Use SIGKILL to terminate\n";
-           (void) !write(STDERR_FILENO ,str, strlen(str));
-           break;
+        case SIGHUP :
+          str = "\naide: received SIGINT, SIGTERM or SIGHUP. cleaning up...\n";
+          (void) !write(STDERR_FILENO ,str, strlen(str));
+          if (conf && (conf->database_out.fp != NULL
+#ifdef WITH_ZLIB
+                  || conf->database_out.gzp != NULL
+#endif
+          )) {
+              if (conf->database_out.url->type == url_file && conf->database_out.flags&DB_FLAG_CREATED) {
+                  str = "remove database_out: file:";
+                  (void) !write(STDERR_FILENO, str, strlen(str));
+                  (void) !write(STDERR_FILENO, conf->database_out.url->value, strlen(conf->database_out.url->value));
+                  str = "\n";
+                  (void) !write(STDERR_FILENO ,str, strlen(str));
+                  unlink(conf->database_out.url->value);
+              }
+          }
+          _exit(SIGNAL_INTERRUPT_ERROR);
+          break;
         case SIGUSR1 :
-           str = "Caught SIGUSR1, toggle debug level\n";
+           str = "\naide: received SIGUSR1, toggle debug level\n";
            (void) !write(STDERR_FILENO ,str, strlen(str));
            toogle_log_level(LOG_LEVEL_DEBUG);
            break;
+        case SIGWINCH :
+           if(ioctl(STDERR_FILENO, TIOCGWINSZ, &winsize) == -1) {
+            conf->progress = 80;
+           } else {
+               conf->progress = winsize.ws_col;
+           }
+        break;
     }
 }
 
@@ -144,7 +153,7 @@ static void sig_handler(int signum)
 
 static void print_version(void)
 {
-  fprintf(stdout, "AIDE %s\n\n", AIDEVERSION );
+  fprintf(stdout, "AIDE %s\n\n", conf->aide_version );
   fprintf(stdout, "Compile-time options:\n%s\n", AIDECOMPILEOPTIONS);
   fprintf(stdout, "Default config values:\n");
   fprintf(stdout, "config file: %s\n", conf->config_file?conf->config_file:"<none>");
@@ -173,6 +182,13 @@ static void print_version(void)
       fprintf(stdout, "%s: %s\n", attributes[hashsums[i].attribute].config_name, ATTR(hashsums[i].attribute)&available_hashsums?"yes":"no");
   }
 
+#ifdef HAVE_FSTYPE
+  fprintf(stdout, "\nAvailable file system type names:\n");
+  for (int i = 0 ; i < num_filesystems; ++i) {
+      fprintf(stdout, "%-20s%s", filesystems[i].str, i%4 == 3 || i+1 == num_filesystems?"\n":"");
+  }
+#endif
+
   fprintf(stdout, "\nDefault compound groups:\n");
   char* predefined_groups[] = { "R", "L", ">", "H", "X" };
   for (unsigned long i = 0 ; i < sizeof(predefined_groups)/sizeof(char*); ++i) {
@@ -219,15 +235,19 @@ static void read_param(int argc,char**argv)
 {
   int i=0;
 
+  enum cmdline_args {
+      ARG_NO_PROGRESS = 1,
+      ARG_LIST        = 2,
+      ARG_NO_COLOR    = 3,
+  };
+
   static struct option options[] =
   {
     { "help", no_argument, NULL, 'h' },
-    { "verbose", optional_argument, NULL, 'V'},
     { "version", no_argument, NULL, 'v'},
     { "config", required_argument, NULL, 'c'},
     { "before", required_argument, NULL, 'B'},
     { "after", required_argument, NULL, 'A'},
-    { "report", no_argument, NULL, 'r'},
     { "init", no_argument, NULL, 'i'},
     { "dry-init", no_argument, NULL, 'n'},
     { "check", no_argument, NULL, 'C'},
@@ -237,7 +257,10 @@ static void read_param(int argc,char**argv)
     { "limit", required_argument, NULL, 'l'},
     { "log-level", required_argument, NULL, 'L'},
     { "workers", required_argument, NULL, 'W'},
+    { "no-progress", no_argument, NULL, ARG_NO_PROGRESS},
+    { "no-color", no_argument, NULL, ARG_NO_COLOR},
     { "compare", no_argument, NULL, 'E'},
+    { "list", no_argument, NULL, ARG_LIST},
     { NULL,0,NULL,0 }
   };
 
@@ -255,9 +278,6 @@ static void read_param(int argc,char**argv)
 	print_version();
 	break;
       }
-      case 'V':{
-        INVALID_ARGUMENT("--verbose", %s, "option no longer supported, use 'log_level' and 'report_level' options instead (see man aide.conf for details)")
-      }
       case 'c':{
 	  conf->config_file=optarg;
       log_msg(LOG_LEVEL_INFO,_("(--config): set config file to '%s'"), conf->config_file);
@@ -287,7 +307,7 @@ static void read_param(int argc,char**argv)
                 }
                 conf->limit_md = pcre2_match_data_create_from_pattern(conf->limit_crx, NULL);
                 if (conf->limit_md == NULL) {
-                    log_msg(LOG_LEVEL_ERROR, "pcre2_match_data_create_from_pattern: failed to allocate memory");
+                    fprintf(stderr, "%s: (%s) pcre2_match_data_create_from_pattern: failed to allocate memory\n", argv[0], "--limit");
                     exit(MEMORY_ALLOCATION_FAILURE);
                 }
 
@@ -295,7 +315,7 @@ static void read_param(int argc,char**argv)
                 if (pcre2_jit < 0) {
                     PCRE2_UCHAR pcre2_error[128];
                     pcre2_get_error_message(pcre2_jit, pcre2_error, 128);
-                    log_msg(LOG_LEVEL_NOTICE, "JIT compilation for limit '%s' failed: %s (fall back to interpreted matching)", conf->limit, pcre2_error);
+                    log_msg(LOG_LEVEL_DEBUG, "JIT compilation for limit '%s' failed: %s (fall back to interpreted matching)", conf->limit, pcre2_error);
                 } else {
                     log_msg(LOG_LEVEL_DEBUG, "JIT compilation for limit '%s' successful", conf->limit);
                 }
@@ -314,54 +334,92 @@ static void read_param(int argc,char**argv)
            break;
                }
       case 'W':{
-#ifdef WITH_PTHREAD
            long num_workers = do_num_workers(optarg);
            if (num_workers < 0) {
                INVALID_ARGUMENT("--workers", invalid number of workers '%s', optarg)
            }
            conf->num_workers = num_workers;
            log_msg(LOG_LEVEL_INFO,"(--workers): set number of workers to %ld (argument value: '%s')", conf->num_workers, optarg);
-#else
-           INVALID_ARGUMENT("--workers", %s, "pthread support not compiled in, recompile AIDE with '--with-pthread'")
-#endif
+           break;
+      }
+      case ARG_NO_PROGRESS:{
+           conf->progress = -1;
+           log_msg(LOG_LEVEL_INFO,"(--no-progress): disable progress bar");
+           break;
+      }
+      case ARG_NO_COLOR:{
+           conf->no_color = false;
+           log_msg(LOG_LEVEL_INFO,"(--no-color): disable colored log output");
            break;
       }
       case 'p':{
             if(conf->action==0){
                 conf->action=DO_DRY_RUN;
                 log_msg(LOG_LEVEL_INFO,"(--path-check): path check command");
-
-                if (strlen(optarg) >= 3 && optarg[1] == ':') {
-                    RESTRICTION_TYPE file_type = get_restriction_from_char(*optarg);
-                    if (file_type == FT_NULL) {
+                if (optarg && *optarg) {
+                    conf->check_file.type = get_f_type_from_char(*optarg);
+                    if (conf->check_file.type == FT_NULL) {
                         INVALID_ARGUMENT("--path-check", invalid file type '%c' (see man aide for details), *optarg)
                     } else {
-                        conf->check_file_type = file_type;
-                        if (optarg[2] != '/') {
-                            INVALID_ARGUMENT("--path-check", '%s' needs to be an absolute path, optarg+2)
-                        } else {
-                            conf->check_path = checked_strdup(optarg+2);
-                            log_msg(LOG_LEVEL_INFO,"(--path-check): set path to '%s' (filetype: %c)", optarg+2, get_restriction_char(conf->check_file_type));
+                        char * tmp = optarg+1;
+#ifdef HAVE_FSTYPE
+                        if (optarg[1] == '=') {
+                            char* colon = strchr(optarg+2, ':');
+                            if (colon) {
+                                char* fs_type_str = NULL;
+                                fs_type_str = checked_strndup(optarg+2, colon-(optarg+2));
+                                if (*fs_type_str) {
+                                    conf->check_file.fs_type = get_fs_type_from_string(fs_type_str);
+                                    if (conf->check_file.fs_type == 0) {
+                                        INVALID_ARGUMENT("--path-check", invalid file system type '%s' (see man aide for details), fs_type_str)
+                                    }
+                                    free(fs_type_str);
+                                    tmp = colon;
+                                } else {
+                                    INVALID_ARGUMENT("--path-check", %s, "missing file system type (see man aide for details)")
+                                }
+                            } else {
+                                INVALID_ARGUMENT("--path-check", %s, "missing ':' (see man aide for details)")
+                            }
                         }
+#endif
+                            if (*tmp == ':') {
+                                char* path = tmp+1;
+                                if (*path) {
+                                    if (*path == '/') {
+                                        conf->check_file.name = checked_strdup(path);
+#ifdef HAVE_FSTYPE
+                                        char* fs_type_str = get_fs_type_string_from_magic(conf->check_file.fs_type);
+                                        log_msg(LOG_LEVEL_INFO,"(--path-check): set path to '%s' (f_type: %c, fs_type: '%s')", path, get_f_type_char_from_f_type(conf->check_file.type), fs_type_str);
+                                        free(fs_type_str);
+#else
+                                        log_msg(LOG_LEVEL_INFO,"(--path-check): set path to '%s' (filetype: %c)", path , get_f_type_char_from_f_type(conf->check_file.type));
+#endif
+                                    } else {
+                                        INVALID_ARGUMENT("--path-check", '%s' needs to be an absolute path, path)
+                                    }
+                                } else {
+                                    INVALID_ARGUMENT("--path-check", %s, "missing path")
+                                }
+                            } else {
+                                INVALID_ARGUMENT("--path-check", %s, "missing ':' (see man aide for details)")
+                            }
                     }
                 } else {
-                    INVALID_ARGUMENT("--path-check", %s, "missing file type or path (see man aide for details)")
+                    INVALID_ARGUMENT("--path-check", %s, "missing file type (see man aide for details)")
                 }
-
             } else {
                 INVALID_ARGUMENT("--path-check", %s, "cannot have multiple commands on a single commandline")
             }
             break;
       }
-      case 'r': {
-       INVALID_ARGUMENT("--report", %s, "option no longer supported, use 'report_url' config option instead (see man aide.conf for detail)")
-      }
       ACTION_CASE("--init", 'i', DO_INIT, "database init")
       ACTION_CASE("--dry-init", 'n', DO_INIT|DO_DRY_RUN, "dry init")
       ACTION_CASE("--check", 'C', DO_COMPARE, "database check")
       ACTION_CASE("--update", 'u', DO_INIT|DO_COMPARE, "database update")
       ACTION_CASE("--compare", 'E', DO_DIFF, "database compare")
       ACTION_CASE("--config-check", 'D', DO_DRY_RUN, "config check")
+      ACTION_CASE("--list", ARG_LIST, DO_LIST, "list")
       default: /* '?' */
 	  exit(INVALID_ARGUMENT_ERROR);
       }
@@ -373,7 +431,7 @@ static void read_param(int argc,char**argv)
   }
 }
 
-static void setdefaults_before_config()
+static void setdefaults_before_config(void)
 {
   DB_ATTR_TYPE X;
 
@@ -397,8 +455,11 @@ static void setdefaults_before_config()
   conf->report_ignore_e2fsattrs = 0UL;
 #endif
 
-  conf->check_path=NULL;
-  conf->check_file_type = FT_REG;
+  conf->check_file.name = NULL;
+  conf->check_file.type = FT_NULL;
+#ifdef HAVE_FSTYPE
+  conf->check_file.fs_type = 0;
+#endif
 
   conf->report_urls=NULL;
   conf->report_level=default_report_options.level;
@@ -412,6 +473,7 @@ static void setdefaults_before_config()
 #endif
       ;
   conf->config_version=NULL;
+  conf->aide_version = AIDEVERSION;
   conf->config_check_warn_unrestricted_rules = false;
   
 #ifdef WITH_ACL
@@ -432,9 +494,9 @@ static void setdefaults_before_config()
   conf->database_in.lineno = 0;
   conf->database_in.fields = NULL;
   conf->database_in.num_fields = 0;
-  conf->database_in.buffer_state = NULL;
   conf->database_in.mdc = NULL;
   conf->database_in.db_line = NULL;
+  conf->database_in.flags = DB_FLAG_NONE;
 
   conf->database_out.url = NULL;
   conf->database_out.filename=NULL;
@@ -447,9 +509,9 @@ static void setdefaults_before_config()
   conf->database_out.lineno = 0;
   conf->database_out.fields = NULL;
   conf->database_out.num_fields = 0;
-  conf->database_out.buffer_state = NULL;
   conf->database_out.mdc = NULL;
   conf->database_out.db_line = NULL;
+  conf->database_out.flags = DB_FLAG_NONE;
 
   conf->database_new.url = NULL;
   conf->database_new.filename=NULL;
@@ -462,21 +524,17 @@ static void setdefaults_before_config()
   conf->database_new.lineno = 0;
   conf->database_new.fields = NULL;
   conf->database_new.num_fields = 0;
-  conf->database_new.buffer_state = NULL;
   conf->database_new.mdc = NULL;
   conf->database_new.db_line = NULL;
+  conf->database_new.flags = DB_FLAG_NONE;
 
-  conf->db_attrs = get_hashes(false);
-  
 #ifdef WITH_ZLIB
   conf->gzip_dbout=0;
 #endif
 
   conf->action=0;
 
-#ifdef WITH_PTHREAD
   conf->num_workers = -1;
-#endif
 
   conf->warn_dead_symlinks=0;
 
@@ -494,6 +552,11 @@ static void setdefaults_before_config()
 
   conf->start_time=time(NULL);
 
+  conf->progress = 0;
+  conf->no_color = true;
+
+  conf->print_details_width = 80;
+
   log_msg(LOG_LEVEL_INFO, "define default attribute definitions");
 
   for (ATTRIBUTE i = 0 ; i < num_attrs ; ++i) {
@@ -522,30 +585,22 @@ static void setdefaults_before_config()
   DB_ATTR_TYPE common_attrs = ATTR(attr_perm)|ATTR(attr_ftype)|ATTR(attr_inode)|ATTR(attr_linkcount)|ATTR(attr_uid)|ATTR(attr_gid);
 
   DB_ATTR_TYPE GROUP_R_HASHES=0LLU;
-#ifdef WITH_MHASH
-  GROUP_R_HASHES=ATTR(attr_md5);
-#endif
-#ifdef WITH_GCRYPT
-  if (gcry_fips_mode_active()) {
-    char* str;
-    log_msg(LOG_LEVEL_NOTICE, "libgcrypt is running in FIPS mode, the following hash(es) are not available: %s", str = diff_attributes(0, ATTR(attr_md5)));
-    free(str);
-  } else {
-    GROUP_R_HASHES = ATTR(attr_md5);
-  }
-#endif
+
+  GROUP_R_HASHES=ATTR(attr_sha3_256);
 
   log_msg(LOG_LEVEL_INFO, "define default groups definitions");
   do_groupdef("R",common_attrs|ATTR(attr_size)|ATTR(attr_linkname)|ATTR(attr_mtime)|ATTR(attr_ctime)|GROUP_R_HASHES|X);
   do_groupdef("L",common_attrs|ATTR(attr_linkname)|X);
   do_groupdef(">",common_attrs|ATTR(attr_size)|ATTR(attr_growing)|ATTR(attr_linkname)|X);
-  do_groupdef("H",get_hashes(false));
+  do_groupdef("H",get_hashes(false)&~DEPRECATED_HASHES);
   do_groupdef("X",X);
   do_groupdef("E",0);
 
+  conf->db_attrs = get_groupval("H");
+
 }
 
-static void setdefaults_after_config()
+static void setdefaults_after_config(void)
 {
   int linenumber=1;
 
@@ -572,40 +627,89 @@ static void setdefaults_after_config()
     conf->action=DO_COMPARE;
   }
 
-#ifdef WITH_PTHREAD
   if(conf->num_workers < 0) {
       conf->num_workers = 1;
       log_msg(LOG_LEVEL_CONFIG, "(default): set 'num_workers' option to %lu", conf->num_workers);
   }
-#endif
 
   if (is_log_level_unset()) {
           set_log_level(LOG_LEVEL_WARNING);
   };
 }
 
+static void list_attribute(db_line* entry, ATTRIBUTE attribute) {
+    char **value = NULL;
+    int num, i, c;
+    int p = conf->print_details_width-5-MAX_WIDTH_DETAILS_STRING;
+
+    DB_ATTR_TYPE attr = ATTR(attribute);
+    const char* name = attributes[attribute].details_string;
+
+    num=get_attribute_values(attr, entry, &value, NULL);
+
+    i = 0;
+    while (i<num) {
+        int olen = strlen(value[i]);
+        int k = 0;
+        while (olen-p*k >= 0) {
+            c = k*(p-1);
+            fprintf(stdout,"  %-*s%c %.*s\n", MAX_WIDTH_DETAILS_STRING, (i+k)?"":name, (i+k)?' ':':', p-1, olen-c>0?&value[i][c]:"");
+            k++;
+        }
+        ++i;
+    }
+    for(i=0; i < num; ++i) { free(value[i]); value[i]=NULL; } free(value); value=NULL;
+}
+
 int main(int argc,char**argv)
 {
   int errorno=0;
 
-#ifdef WITH_PTHREAD
-  log_init();
-#endif
-
 #ifdef WITH_LOCALE
   setlocale(LC_ALL,"");
   bindtextdomain(PACKAGE,LOCALEDIR);
   textdomain(PACKAGE);
 #endif
   umask(0177);
-  init_sighandler();
-  init_crypto_lib();
+
+  init_db_sighandler();
+
+  init_hashsum_lib();
 
   setdefaults_before_config();
 
   log_msg(LOG_LEVEL_INFO, "read command line parameters");
   read_param(argc,argv);
 
+  int stderr_isatty = isatty(STDERR_FILENO);
+  if (stderr_isatty == 0) {
+        log_msg(LOG_LEVEL_DEBUG, "isatty() failed for 'STDERR_FILENO': %s", strerror(errno));
+  }
+
+  if (conf->no_color) {
+      if (stderr_isatty) {
+          log_msg(LOG_LEVEL_DEBUG, "enable colored log output (stderr refers to a terminal)");
+      } else {
+          conf->no_color = false;
+          log_msg(LOG_LEVEL_INFO, "disable colored log output (stderr does not refer to a terminal)");
+      }
+  }
+  set_colored_log(conf->no_color);
+
+  if (!(conf->action&DO_DRY_RUN) && !(conf->action&DO_LIST)) {
+      if (conf->progress >= 0) {
+          if (stderr_isatty) {
+              log_msg(LOG_LEVEL_DEBUG, "enable progress bar (stderr refers to a terminal)");
+              if (progress_start()) {
+                  log_msg(LOG_LEVEL_DEBUG, "initialize signal handler for SIGWINCH");
+                  signal(SIGWINCH,sig_handler);
+              }
+          } else {
+              log_msg(LOG_LEVEL_INFO, "disable progress bar (stderr does not refer to a terminal)");
+          }
+      }
+  }
+
   /* get hostname */
   conf->hostname = checked_malloc(sizeof(char) * MAXHOSTNAMELEN + 1);
   if (gethostname(conf->hostname,MAXHOSTNAMELEN) == -1) {
@@ -617,6 +721,7 @@ int main(int argc,char**argv)
   }
 
   log_msg(LOG_LEVEL_INFO, "parse configuration");
+  progress_status(PROGRESS_CONFIG, NULL);
   errorno=parse_config(before, conf->config_file, after);
   if (errorno==RETFAIL){
     exit(INVALID_CONFIGURELINE_ERROR);
@@ -626,40 +731,41 @@ int main(int argc,char**argv)
 
   setdefaults_after_config();
 
+  log_msg(LOG_LEVEL_DEBUG, "initialize signal handler for SIGUSR1");
+  signal(SIGUSR1,sig_handler);
+
   log_msg(LOG_LEVEL_CONFIG, "report_urls:");
   log_report_urls(LOG_LEVEL_CONFIG);
 
   log_msg(LOG_LEVEL_RULE, "rule tree:");
   log_tree(LOG_LEVEL_RULE, conf->tree, 0);
 
-  if (conf->action&DO_INIT
-          && (conf->tree)->childs == NULL
-          && (conf->tree)->equ_rx_lst == NULL
-          && (conf->tree)->sel_rx_lst == NULL
-          && (conf->tree)->neg_rx_lst == NULL
-     ) {
+  if (conf->action&DO_INIT && is_tree_empty(conf->tree)) {
       log_msg(LOG_LEVEL_WARNING, "rule tree is empty, no files will be added to the database");
   }
 
-  if (conf->check_path) {
-      rx_rule* rule = NULL;
-      match_result match = check_rxtree(conf->check_path, conf->tree, &rule, conf->check_file_type, "disk (path-check)");
-      print_match(conf->check_path, rule, match, conf->check_file_type);
-      switch (match) {
+  if (conf->check_file.name) {
+      match_t path_match = check_rxtree(conf->check_file, conf->tree, "disk (path-check)", true, NULL);
+      print_match(conf->check_file, path_match);
+      switch (path_match.result) {
           case RESULT_PARTIAL_LIMIT_MATCH:
           case RESULT_NO_LIMIT_MATCH:
               exit(2);
           case RESULT_EQUAL_MATCH:
           case RESULT_SELECTIVE_MATCH:
               exit(0);
-          case RESULT_NO_MATCH:
+          case RESULT_NON_RECURSIVE_NEGATIVE_MATCH:
+          case RESULT_RECURSIVE_NEGATIVE_MATCH:
+          case RESULT_NEGATIVE_PARENT_MATCH:
+          case RESULT_NO_RULE_MATCH:
           case RESULT_PARTIAL_MATCH:
+          case RESULT_PART_LIMIT_AND_NO_RECURSE_MATCH:
               exit(1);
       }
   }
 
   /* Let's do some sanity checks for the config */
-  if (conf->action&(DO_DIFF|DO_COMPARE) && !(conf->database_in.url)) {
+  if (conf->action&(DO_DIFF|DO_COMPARE|DO_LIST) && !(conf->database_in.url)) {
     log_msg(LOG_LEVEL_ERROR,_("missing 'database_in', config option is required"));
     exit(INVALID_ARGUMENT_ERROR);
   }
@@ -696,6 +802,39 @@ int main(int argc,char**argv)
       exit (0);
   }
 
+  if ((conf->action&DO_LIST)) {
+      if(db_init(&(conf->database_in), true, false)==RETFAIL) {
+          exit(IO_ERROR);
+      }
+      log_msg(LOG_LEVEL_INFO, "list entries from database: %s", (conf->database_in.url)->raw);
+      db_entry_t entry;
+      while((entry = db_readline(&(conf->database_in), false)).line != NULL) {
+          log_msg(LOG_LEVEL_RULE, "\u252c process '%s' (filetype: %c)", (entry.line)->filename, get_f_type_char_from_perm((entry.line)->perm));
+          fprintf(stdout, "%s\n", (entry.line)->filename);
+          for (int j=0; j < report_attrs_order_length; ++j) {
+              switch(report_attrs_order[j]) {
+                  case attr_allhashsums:
+                      for (int i = 0 ; i < num_hashes ; ++i) {
+                          if (ATTR(hashsums[i].attribute)&(entry.line)->attr) { list_attribute((entry.line), hashsums[i].attribute); }
+                      }
+                      break;
+                  case attr_size:
+                      if (ATTR(attr_size)&(entry.line)->attr) { list_attribute((entry.line), attr_size); }
+                      if (ATTR(attr_sizeg)&(entry.line)->attr) { list_attribute((entry.line), attr_sizeg); }
+                      break;
+                  default:
+                      if (ATTR(report_attrs_order[j])&(entry.line)->attr) { list_attribute((entry.line), report_attrs_order[j]); }
+                      break;
+              }
+          }
+          list_attribute((entry.line), attr_attr);
+          fprintf(stdout, "\n");
+          log_msg(LOG_LEVEL_RULE, "\u2534 finish processing '%s'", (entry.line)->filename);
+      }
+      db_close();
+      exit (0);
+  }
+
   if (!(conf->action&DO_DRY_RUN)) {
 
   if (!init_report_urls()) {
@@ -735,27 +874,14 @@ int main(int argc,char**argv)
 	exit(IO_ERROR);
     }
 
-#ifdef WITH_PTHREAD
-    if((conf->action&DO_INIT || conf->action&DO_COMPARE) && conf->num_workers){
-      if(db_disk_start_threads()==RETFAIL)
-          exit(THREAD_ERROR);
-    }
-#endif
-
-    log_msg(LOG_LEVEL_INFO, "populate tree");
     populate_tree(conf->tree);
 
-#ifdef WITH_PTHREAD
-    if((conf->action&DO_INIT || conf->action&DO_COMPARE) && conf->num_workers){
-      if(db_disk_finish_threads() == RETFAIL)
-          exit(THREAD_ERROR);
-    }
-#endif
-
     if(conf->action&DO_INIT) {
-        log_msg(LOG_LEVEL_INFO, "write new entries to database: %s:%s", get_url_type_string((conf->database_out.url)->type), (conf->database_out.url)->value);
+        progress_status(PROGRESS_WRITEDB, NULL);
+        log_msg(LOG_LEVEL_INFO, "write new entries to database: %s", (conf->database_out.url)->raw);
         write_tree(conf->tree);
     }
+    progress_stop();
 
     db_close();
 
