@@ -1,7 +1,7 @@
 /*
  * AIDE (Advanced Intrusion Detection Environment)
  *
- * Copyright (C) 1999-2006, 2010-2011, 2013, 2015-2016, 2019-2023 Rami Lehti,
+ * Copyright (C) 1999-2006, 2010-2011, 2013, 2015-2016, 2019-2025 Rami Lehti,
  *               Pablo Virolainen, Richard van den Berg, Hannes von Haugwitz
  *
  * This program is free software; you can redistribute it and/or
@@ -29,10 +29,10 @@
 #include "attributes.h"
 #include "conf_ast.h"
 #include "config.h"
+#include "errorcodes.h"
 #include "hashsum.h"
 #include "list.h"
 #include "rx_rule.h"
-#include "seltree_struct.h"
 #include "url.h"
 #include "commandconf.h"
 #include "conf_lex.h"
@@ -55,11 +55,9 @@
 #ifdef WITH_E2FSATTRS
 #include "e2fsattrs.h"
 #endif
-#ifdef WITH_PTHREAD
 #include <math.h>
 #include <errno.h>
 #include <unistd.h>
-#endif
 
 #define BUFSIZE 4096
 #define ZBUFSIZE 16384
@@ -155,6 +153,8 @@ url_t* parse_url(char* val, int linenumber, char* filename, char* linebuf)
 
   free(val_copy);
 
+  u->raw = checked_strdup(val);
+
   return u;
 }
 
@@ -201,76 +201,7 @@ int parse_config(char *before, char *config, char* after) {
 int conf_input_wrapper(char* buf, int max_size, FILE* in)
 {
   int retval=0;
-
-  /* FIXME Add support for gzipped config. :) */
   retval=fread(buf,1,max_size,in);
-
-  return retval;
-}
-
-int db_input_wrapper(char* buf, int max_size, database* db)
-{
-  log_msg(LOG_LEVEL_TRACE,"db_input_wrapper(): parameters: buf=%p, max_size=%d, db=%p)", buf, max_size, db);
-  int retval=0;
-#ifdef WITH_ZLIB
-  int c;
-#endif
-
-#ifdef WITH_CURL
-  switch ((db->url)->type) {
-  case url_http:
-  case url_https:
-  case url_ftp: {
-    retval=url_fread(buf,1,max_size,(URL_FILE *)db->fp);
-    if (db->mdc) {
-        update_md(db->mdc, buf, retval);
-    }
-    break;
-  } 
-  default:
-#endif /* WITH CURL */
-
-#ifdef WITH_ZLIB
-  if (db->gzp!=NULL) {
-    c=gzgetc(db->gzp);
-    retval= (c==EOF) ? 0 : (buf[0] = c,1);
-  }
-  if (db->gzp==NULL) {
-    c=fgetc(db->fp);
-    if(c==(unsigned char)'\037'){
-      c=fgetc(db->fp);
-      if(c==(unsigned char)'\213'){
-    log_msg(LOG_LEVEL_DEBUG,"db_input_wrapper(): handle gzip header");
-    lseek(fileno((FILE *)db->fp),0L,SEEK_SET);
-    db->gzp=gzdopen(fileno((FILE *)db->fp),"rb");
-    c=gzgetc(db->gzp);
-    log_msg(LOG_LEVEL_DEBUG, "db_input_wrapper(): first character after gzip header is: %c(%#X)\n",c,c);
-  if(c==-1) {
-    int xx;
-      log_msg(LOG_LEVEL_ERROR,"reading gzipped file failed: %s", gzerror(db->gzp,&xx));
-    exit(EXIT_FAILURE);
-  }
-      }else {
-       /* False alarm */
-       ungetc(c,db->fp);
-      }
-    }
-    retval= (c==EOF) ? 0 : (buf[0] = c,1);
-  }
-
-#else /* WITH_ZLIB */
-  retval=fread(buf,1,max_size,db->fp);
-#endif /* WITH_ZLIB */
-
-  if (db->mdc) {
-      update_md(db->mdc, buf, retval);
-  }
-
-
-#ifdef WITH_CURL
-  }
-#endif /* WITH CURL */
-  log_msg(LOG_LEVEL_TRACE,"db_input_wrapper(): return value: %d", retval);
   return retval;
 }
 
@@ -310,7 +241,7 @@ void do_undefine(char* name, int linenumber, char* filename, char* linebuf)
   }
 }
 
-bool add_rx_rule_to_tree(char* rx, char* rule_prefix, RESTRICTION_TYPE restriction, DB_ATTR_TYPE attr, int type, seltree *tree, int linenumber, char* filename, char* linebuf) {
+bool add_rx_rule_to_tree(char* rx, char* rule_prefix, rx_restriction_t restriction, DB_ATTR_TYPE attr, int type, seltree *tree, int linenumber, char* filename, char* linebuf) {
 
     rx_rule* r=NULL;
 
@@ -320,6 +251,7 @@ bool add_rx_rule_to_tree(char* rx, char* rule_prefix, RESTRICTION_TYPE restricti
     char *rs_str = NULL;
 
     char *regex;
+    char *node_path = NULL;
     if (rule_prefix) {
         int length = strlen(rule_prefix)+strlen(rx);
         regex = checked_malloc(length+1);
@@ -330,22 +262,12 @@ bool add_rx_rule_to_tree(char* rx, char* rule_prefix, RESTRICTION_TYPE restricti
         regex = checked_strdup(rx);
     }
 
-    if ((r = add_rx_to_tree(regex, restriction, type, tree, linenumber, filename, linebuf)) == NULL) {
+    if ((r = add_rx_to_tree(regex, restriction, type, tree, linenumber, filename, linebuf, &node_path)) == NULL) {
         retval = false;
     }else {
-        r->config_linenumber = linenumber;
-        r->config_filename = filename;
-        r->config_line = checked_strdup(linebuf);
         r->prefix = rule_prefix;
 
-        char *str;
-
-        DB_ATTR_TYPE unsupported_hashes = attr&(get_hashes(true)&~get_hashes(false));
-        if (unsupported_hashes) {
-            LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_WARNING, "ignoring unsupported hash algorithm(s): %s", str = diff_attributes(0, unsupported_hashes));
-            free(str);
-            attr &= ~unsupported_hashes;
-        }
+        attr &= validate_hashes(attr, linenumber, filename, linebuf);
 
         DB_ATTR_TYPE unsupported_attrs = attr&
             (0
@@ -363,10 +285,14 @@ bool add_rx_rule_to_tree(char* rx, char* rule_prefix, RESTRICTION_TYPE restricti
 #endif
 #ifndef WITH_CAPABILITIES
              |ATTR(attr_capabilities)
+#endif
+#ifndef HAVE_FSTYPE
+             |ATTR(attr_fs_type)
 #endif
             )
             ;
         if (unsupported_attrs) {
+            char *str;
             LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_WARNING, "ignoring not compiiled-in attribute(s): %s", str = diff_attributes(0, unsupported_attrs));
             free(str);
             attr &= ~unsupported_attrs;
@@ -374,14 +300,14 @@ bool add_rx_rule_to_tree(char* rx, char* rule_prefix, RESTRICTION_TYPE restricti
 
         r->attr=attr;
         if (attr&ATTR(attr_sizeg)) {
-            log_msg(LOG_LEVEL_NOTICE, "%s:%d: Using 'S' attribute is DEPRECATED and will be removed in the release after next. Update your config and use 'growing+s' instead (line: '%s')", filename, linenumber, linebuf);
+            log_msg(LOG_LEVEL_WARNING, "%s:%d: Using 'S' attribute is DEPRECATED and will be removed in the release after next. Update your config and use 'growing+s' instead (line: '%s')", filename, linenumber, linebuf);
         }
         if (attr&ATTR(attr_compressed) && !(attr&get_hashes(false))) {
             log_msg(LOG_LEVEL_WARNING, "%s:%d: ignore 'comprressed' attribute (no hashsum attributes are set) (line: '%s')", filename, linenumber, linebuf);
         }
         conf->db_out_attrs |= attr;
 
-        LOG_CONFIG_FORMAT_LINE_PREFIX(LOG_LEVEL_CONFIG, "add %s '%s%s %s %s' to node '%s'", get_rule_type_long_string(type), get_rule_type_char(type), r->rx, rs_str = get_restriction_string(r->restriction), attr_str = diff_attributes(0, r->attr),  (r->node)->path)
+        LOG_CONFIG_FORMAT_LINE_PREFIX(LOG_LEVEL_CONFIG, "add %s '%s%s %s %s' to node '%s'", get_rule_type_long_string(type), get_rule_type_char(type), r->rx, rs_str = get_restriction_string(r->restriction), attr_str = diff_attributes(0, r->attr), node_path)
         free(rs_str);
         free(attr_str);
 
@@ -492,27 +418,23 @@ bool do_dbdef(DB_TYPE dbtype ,char* val, int linenumber, char* filename, char* l
     db->linenumber = linenumber;
     db->filename = filename;
     db->linebuf = linebuf?checked_strdup(linebuf):NULL;
-    LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_CONFIG, "set '%s' option to '%s:%s'", db_option_name, get_url_type_string(u->type), u->value)
+    LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_CONFIG, "set '%s' option to '%s'", db_option_name, u->raw)
     } else {
         return false;
     }
   } else {
-    LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_NOTICE, "'%s' option already set to '%s:%s' (ignore new value '%s')", db_option_name, get_url_type_string((db->url)->type), (db->url)->value, val);
+    LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_NOTICE, "'%s' option already set to '%s' (ignore new value '%s')", db_option_name, (db->url)->raw, val);
   }
   return true;
 }
 
 bool do_repurldef(char* val, int linenumber, char* filename, char* linebuf) {
     url_t* u = parse_url(val, linenumber, filename, linebuf);
-    if (add_report_url(u, linenumber, filename, linebuf)) {
-        LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_CONFIG, "set 'report_url' to '%s%s%s'", get_url_type_string(u->type), u->value?":":"", u->value?u->value:"")
-            return true;
-    }
-    return false;
+    return add_report_url(u, linenumber, filename, linebuf);
 }
 
 bool do_reportlevel(char* val, int linenumber, char* filename, char* linebuf) {
-  REPORT_LEVEL report_level=0;
+  REPORT_LEVEL report_level = REPORT_LEVEL_UNKNOWN;
 
   report_level = get_report_level(val);
   if (report_level) {
@@ -541,7 +463,6 @@ bool do_rootprefix(char* val, int linenumber, char* filename, char* linebuf) {
     }
 }
 
-#ifdef WITH_PTHREAD
 long do_num_workers(const char *str) {
     char *err;
     long number = strtol(str,&err,10);
@@ -557,7 +478,6 @@ long do_num_workers(const char *str) {
     }
     return number;
 }
-#endif
 
 #ifdef WITH_E2FSATTRS
 void do_report_ignore_e2fsattrs(char* val, int linenumber, char* filename, char* linebuf) {
