@@ -1,7 +1,7 @@
 /*
  * AIDE (Advanced Intrusion Detection Environment)
  *
- * Copyright (C) 2019-2023 Hannes von Haugwitz
+ * Copyright (C) 2019-2025 Hannes von Haugwitz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -25,6 +25,8 @@
 #include <stdio.h>
 #include <string.h>
 #include <sys/stat.h>
+#define PCRE2_CODE_UNIT_WIDTH 8
+#include <pcre2.h>
 #include "attributes.h"
 #include "conf_ast.h"
 #include "db_config.h"
@@ -53,7 +55,7 @@
 #include <errno.h>
 #include <unistd.h>
 
-LOG_LEVEL eval_log_level = LOG_LEVEL_DEBUG;
+LOG_LEVEL eval_log_level = LOG_LEVEL_TRACE;
 
 bool log_level_set_in_config = false;
 
@@ -96,6 +98,9 @@ static char* eval_string_expression(struct string_expression* expression, int li
             if ((entry = list_find(expression->str, conf->defsyms))) {
                 str = checked_strdup(((symba*)entry->data)->value?((symba*)entry->data)->value:"");
                 LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_CONFIG, "variable substitution: replace '@@%c%s%c' with '%s'", '{', expression->str, '}', str)
+            } else if (strcmp(expression->str, "AIDE_VERSION") == 0) {
+                str = checked_strdup(conf->aide_version);
+                LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_CONFIG, "variable substitution: replace '@@%c%s%c' with '%s'", '{', expression->str, '}', str)
             } else if (strcmp(expression->str, "HOSTNAME") == 0 && conf->hostname) {
                 str = checked_strdup(conf->hostname);
                 LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_CONFIG, "variable substitution: replace '@@%c%s%c' with '%s'", '{', expression->str, '}', str)
@@ -111,7 +116,7 @@ static char* eval_string_expression(struct string_expression* expression, int li
             str = checked_malloc(length+1);
             strncpy(str, left, length+1);
             strncat(str, right, length-strlen(left)+1);
-            log_msg(eval_log_level, "eval(%p): string concat '%s' + '%s' evaluates to %s", expression, left, right, str);
+            log_msg(eval_log_level, "eval(%p): string concat '%s' + '%s' evaluates to %s", (void*) expression, left, right, str);
             free(left);
             free(right);
             break;
@@ -145,7 +150,7 @@ static DB_ATTR_TYPE eval_attribute_expression(struct attribute_expression* expre
             LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_ERROR, "group '%s' is not defined", expression->right)
                 exit(INVALID_CONFIGURELINE_ERROR);
         }
-        log_msg(eval_log_level, "eval(%p): attribute group '%s' evaluates to %llu", expression, expression->right, attr_r);
+        log_msg(eval_log_level, "eval(%p): attribute group '%s' evaluates to %llu", (void*) expression, expression->right, attr_r);
         switch (expression->op) {
             case ATTR_OP_GROUP:
                 attr = attr_r;
@@ -157,9 +162,9 @@ static DB_ATTR_TYPE eval_attribute_expression(struct attribute_expression* expre
                 attr &= (~attr_r);
                 break;
         }
-        log_msg(eval_log_level, "eval(%p): attribute expression (op: %d, left: %p, right: '%s') evaluates to %llu", expression, expression->op, expression->left, expression->right, attr);
+        log_msg(eval_log_level, "eval(%p): attribute expression (op: %d, left: %p, right: '%s') evaluates to %llu", (void*) expression, expression->op, (void*) expression->left, expression->right, attr);
     } else {
-        log_msg(eval_log_level, "eval(%p): attribute expression is NULL and evaluates to %llu", expression, attr);
+        log_msg(eval_log_level, "eval(%p): attribute expression is NULL and evaluates to %llu", (void*) expression, attr);
     }
     return attr;
 }
@@ -173,12 +178,7 @@ static void set_database_attr_option(DB_ATTR_TYPE attr, int linenumber, char *fi
             free(str);
             exit(INVALID_CONFIGURELINE_ERROR);
         }
-        DB_ATTR_TYPE unsupported_hashes = attr&(get_hashes(true)&~get_hashes(false));
-        if (unsupported_hashes) {
-            LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_WARNING, "ignoring unsupported hash algorithm(s): %s", str = diff_attributes(0, unsupported_hashes));
-            free(str);
-            attr &= ~unsupported_hashes;
-        }
+        attr &= validate_hashes(attr, linenumber, filename, linebuf);
         LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_CONFIG, "set 'database_attrs' option to: %s", str = diff_attributes(0, attr));
         free(str);
         conf->db_attrs = attr;
@@ -262,7 +262,7 @@ static void eval_config_statement(config_option_statement statement, int linenum
         case REPORT_FORMAT_OPTION:
             str = eval_string_expression(statement.e, linenumber, filename, linebuf);
             REPORT_FORMAT report_format = get_report_format(str);
-            if (report_format) {
+            if (report_format != REPORT_FORMAT_UNKNOWN) {
                 conf->report_format = report_format;
                 LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_CONFIG, "set 'report_format' option to '%s' (raw: %d)", str, report_format)
             } else {
@@ -297,15 +297,10 @@ static void eval_config_statement(config_option_statement statement, int linenum
             conf->config_version = str;
             LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_CONFIG, "set 'config_version' option to '%s'", str)
             break;
-        case VERBOSE_OPTION:
-            log_msg(LOG_LEVEL_ERROR, "%s:%d: 'verbose' option is no longer supported, use 'log_level' and 'report_level' options instead (see man aide.conf for details) (line: '%s')", conf_filename, conf_linenumber, conf_linebuf);
-            exit(INVALID_CONFIGURELINE_ERROR);
-            break;
         case LIMIT_CMDLINE_OPTION:
             /* command-line options are ignored here */
             break;
         case NUM_WORKERS:
-#ifdef WITH_PTHREAD
             str = eval_string_expression(statement.e, linenumber, filename, linebuf);
 
             if (conf->num_workers < 0) {
@@ -319,47 +314,98 @@ static void eval_config_statement(config_option_statement statement, int linenum
             } else {
                     LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_NOTICE, "'num_workers' option already set (ignore new value '%s')", str)
             }
-#else
-            LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_ERROR, "%s", "pthreads support not compiled in, recompile AIDE with '--with-pthread'")
-            exit(INVALID_CONFIGURELINE_ERROR);
-#endif
             break;
     }
 }
 
+static void parse_version(const char * version_str, long version[3], int linenumber, char *filename, char* linebuf) {
+    char * endptr = NULL;
+    int i = 0;
+
+    const char *ptr = version_str;
+    while (*ptr && i < 3) {
+        version[i] = strtol(ptr, &endptr, 10);
+        if ( ptr == endptr || errno==ERANGE || version[i] < 0) {
+            LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_ERROR, "invalid version string: '%s', expected MAJOR.MINOR.PATCH", version_str);
+            exit(INVALID_CONFIGURELINE_ERROR);
+        }
+        if (*endptr != '.') {
+            break;
+        }
+        ptr = endptr + 1;
+        i++;
+    }
+    if (endptr && *endptr) {
+        LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_DEBUG, "ignore suffix '%s' of version string '%s'", endptr, version_str);
+    }
+
+    LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_DEBUG, "parse version string '%s' to major: %ld, minor: %ld, patch: %ld version", version_str, version[0], version[1], version[2] );
+}
+
+static bool compare_version_ge(const char *left_version_str, const char *right_version_str, int linenumber, char *filename, char* linebuf) {
+    long left_version[3] = { 0 };
+    long right_version[3] = { 0 };
+    parse_version(left_version_str, left_version, linenumber, filename, linebuf);
+    parse_version(right_version_str, right_version, linenumber, filename, linebuf);
+
+    for (int i = 0 ; i <= 2 ; ++i) {
+        if (left_version[i] < right_version[i]) {
+            return false;
+        } else if (left_version[i] > right_version[i]) {
+            return true;
+        }
+    }
+    return true;
+}
+
 static bool eval_bool_expression(struct bool_expression* expression, int linenumber, char *filename, char* linebuf) {
-    bool result = false, left;
-    char * str;
+    bool result = false;
+    union {
+        bool _bool;
+        char *_str;
+    } left;
+    union {
+        bool _bool;
+        char *_str;
+    } right;
 
     switch (expression->op) {
         case BOOL_OP_EXISTS:
-            str = eval_string_expression(expression->expr, linenumber, filename, linebuf);
-            int retval = access(str, F_OK);
+            left._str = eval_string_expression(expression->left._str, linenumber, filename, linebuf);
+            int retval = access(left._str, F_OK);
             result = (retval == 0);
-            log_msg(LOG_LEVEL_DEBUG, "access('%s', F_OK) returns %d: (%s)", str, retval, result?"Success":strerror(errno));
-            log_msg(eval_log_level, "eval(%p): bool exists '%s': %s", expression, str, btoa(result));
-            free(str);
+            log_msg(LOG_LEVEL_DEBUG, "access('%s', F_OK) returns %d: (%s)", left._str, retval, result?"Success":strerror(errno));
+            log_msg(eval_log_level, "eval(%p): bool exists '%s': %s", (void*) expression, left._str, btoa(result));
+            free(left._str);
             break;
         case BOOL_OP_DEFINED:
-            str = eval_string_expression(expression->expr, linenumber, filename, linebuf);
-            result = list_find(str, conf->defsyms) != NULL;
-            log_msg(eval_log_level, "eval(%p): bool defined '%s': %s", expression, str, btoa(result));
-            free(str);
+            left._str = eval_string_expression(expression->left._str, linenumber, filename, linebuf);
+            result = list_find(left._str, conf->defsyms) != NULL;
+            log_msg(eval_log_level, "eval(%p): bool defined '%s': %s", (void*) expression, left._str, btoa(result));
+            free(left._str);
             break;
         case BOOL_OP_HOSTNAME:
-            str = eval_string_expression(expression->expr, linenumber, filename, linebuf);
+            left._str = eval_string_expression(expression->left._str, linenumber, filename, linebuf);
             if (conf->hostname) {
-                result = strcmp(str, conf->hostname) == 0;
+                result = strcmp(left._str, conf->hostname) == 0;
             } else {
                 LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_WARNING, "%s", "hostname not available; ifhost and ifnhost always evaluate to 'false'")
             }
-            log_msg(eval_log_level, "eval(%p): bool hostname '%s' (hostname: '%s'): %s", expression, str, conf->hostname, btoa(result));
-            free(str);
+            log_msg(eval_log_level, "eval(%p): bool hostname '%s' (hostname: '%s'): %s", (void*) expression, left._str, conf->hostname, btoa(result));
+            free(left._str);
             break;
         case BOOL_OP_NOT:
-            left = eval_bool_expression(expression->left, linenumber, filename, linebuf);
-            result = !left;
-            log_msg(eval_log_level, "eval(%p): bool !%s: %s", expression, btoa(left), btoa(result));
+            left._bool = eval_bool_expression(expression->left._bool, linenumber, filename, linebuf);
+            result = !left._bool;
+            log_msg(eval_log_level, "eval(%p): bool !%s: %s", (void*) expression, btoa(left._bool), btoa(result));
+            break;
+        case BOOL_OP_VERSION_GE:
+            left._str = eval_string_expression(expression->left._str, linenumber, filename, linebuf);
+            right._str = eval_string_expression(expression->right._str, linenumber, filename, linebuf);
+            result = compare_version_ge(left._str, right._str, linenumber, filename, linebuf);
+            log_msg(eval_log_level, "eval(%p): bool %s version_ge %s: %s", (void*) expression, left._str, right._str, btoa(result));
+            free(left._str);
+            free(right._str);
             break;
     }
     return result;
@@ -380,7 +426,7 @@ static void eval_group_statement(group_statement statement, int linenumber, char
 }
 
 static bool evaL_if_condition(if_condition* c) {
-    log_msg(eval_log_level, "eval(%p): if condition", c);
+    log_msg(eval_log_level, "eval(%p): if condition", (void*) c);
     bool cond_result = eval_bool_expression(c->expression, c->linenumber, c->filename, c->linebuf);
     log_msg(LOG_LEVEL_CONFIG,"%s:%d: if condition results to '%s' (line: '%s')", c->filename, c->linenumber, btoa(cond_result), c->linebuf);
     return cond_result;
@@ -390,12 +436,12 @@ static void eval_if_statement(if_statement statement, int include_depth, char* r
     bool condition = evaL_if_condition(statement.condition);
 
     if (condition) {
-        log_msg(eval_log_level, "eval(%p): if branch", statement.if_branch);
+        log_msg(eval_log_level, "eval(%p): if branch", (void*) statement.if_branch);
         if (statement.if_branch) {
             eval_config(statement.if_branch, include_depth, rule_prefix);
         }
     } else {
-        log_msg(eval_log_level, "eval(%p): else branch", statement.else_branch);
+        log_msg(eval_log_level, "eval(%p): else branch", (void*) statement.else_branch);
         if (statement.else_branch) {
             eval_config(statement.else_branch, include_depth, rule_prefix);
         }
@@ -485,7 +531,7 @@ static void include_file(const char* file, bool execute, int include_depth, char
             char* config_str = pipe2string(p_stdout[0]);
             close(p_stdout[0]);
             char* child_stderr = pipe2string(p_stderr[0]);
-            close(p_stdout[0]);
+            close(p_stderr[0]);
 
             int wstatus;
             waitpid(pid, &wstatus, 0);
@@ -495,7 +541,7 @@ static void include_file(const char* file, bool execute, int include_depth, char
                 while (child_stderr && *child_stderr != '\0') {
                     newline = strchr(child_stderr, '\n');
                     if (newline != NULL) {
-                        log_msg(LOG_LEVEL_ERROR, "%s: stderr> %.*s", file, newline-child_stderr, child_stderr);
+                        log_msg(LOG_LEVEL_ERROR, "%s: stderr> %.*s", file, (int)(newline-child_stderr), child_stderr);
                         child_stderr = newline+1;
                     } else {
                         log_msg(LOG_LEVEL_ERROR, "%s: stderr> %s", file, child_stderr);
@@ -554,7 +600,7 @@ static void include_directory(const char* dir, const char* rx, bool execute, cha
     if((crx = pcre2_compile((PCRE2_SPTR) rx, PCRE2_ZERO_TERMINATED, PCRE2_UTF, &pcre2_errorcode, &pcre2_erroffset, NULL)) == NULL) {
         PCRE2_UCHAR pcre2_error[128];
         pcre2_get_error_message(pcre2_errorcode, pcre2_error, 128);
-        LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_ERROR, "'%s': error in regular expression '%s' at %i: %s", execute?"@@x_include":"@@include", rx, pcre2_erroffset, pcre2_error)
+        LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_ERROR, "'%s': error in regular expression '%s' at %zu: %s", execute?"@@x_include":"@@include", rx, pcre2_erroffset, pcre2_error)
         exit(INVALID_CONFIGURELINE_ERROR);
     }
     pcre2_match_data *match_data = pcre2_match_data_create_from_pattern(crx, NULL);
@@ -668,29 +714,82 @@ static void eval_include_statement(include_statement statement, int include_dept
     }
 }
 
-static RESTRICTION_TYPE eval_restriction_expression(restriction_expression *expression, int linenumber, char *filename, char* linebuf) {
-    RESTRICTION_TYPE rs = FT_NULL, rs_r;
+static FT_TYPE eval_ft_restriction_expression(ft_restriction_expression *expression, int linenumber, char *filename, char* linebuf) {
+    FT_TYPE rs = FT_NULL, rs_r;
 
     if (expression) {
-        char *unrestricted = "0";
-        if (strncmp(expression->right, unrestricted, strlen(unrestricted)) == 0) {
-            rs_r = FT_NULL;
-            log_msg(eval_log_level, "eval(%p): restriction '%s', evaluates to %d", expression, unrestricted, rs);
-        } else {
-            rs_r = (strlen(expression->right) == 1)?get_restriction_from_char(*(expression->right)):FT_NULL;
-            log_msg(eval_log_level, "eval(%p): restriction file type '%s' evaluates to %d", expression, expression->right, rs_r);
-            if (rs_r == FT_NULL) {
-                LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_ERROR, "invalid restriction '%s'", expression->right)
-                    exit(INVALID_CONFIGURELINE_ERROR);
-            }
+        rs_r = (strlen(expression->right) == 1)?get_f_type_from_char(*(expression->right)):FT_NULL;
+        log_msg(eval_log_level, "eval(%p): file type restriction '%s' evaluates to %u", (void*) expression, expression->right, rs_r);
+        if (rs_r == FT_NULL) {
+            LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_ERROR, "invalid restriction '%s'", expression->right)
+                exit(INVALID_CONFIGURELINE_ERROR);
         }
         if (expression->left == NULL) {
             rs = rs_r;
         } else {
-            rs = eval_restriction_expression(expression->left, linenumber, filename, linebuf) | rs_r;
+            rs = eval_ft_restriction_expression(expression->left, linenumber, filename, linebuf) | rs_r;
         }
     } else {
-        log_msg(eval_log_level, "eval(%p): restriction is NULL, returning %d", expression, rs);
+        log_msg(eval_log_level, "eval(%p): file type restriction is NULL, returning %u", (void*) expression, rs);
+    }
+    return rs;
+}
+
+static rx_restriction_t eval_restriction_expression(restriction_expression *expression, int linenumber, char *filename, char* linebuf) {
+    rx_restriction_t rs = { .f_type = FT_NULL,
+#ifdef HAVE_FSTYPE
+        .fs_type = 0
+#endif
+    };
+
+    if (expression) {
+        if (
+            expression->fs_type == NULL &&
+            expression->f_types == NULL) {
+            log_msg(eval_log_level, "eval(%p): restriction is '0', returning {"
+#ifdef HAVE_FSTYPE
+                    "fs_type 0x%lx, "
+#endif
+                    "file type: %u)", (void*) expression,
+#ifdef HAVE_FSTYPE
+                    rs.fs_type,
+#endif
+                    rs.f_type);
+        } else {
+            rs.f_type = eval_ft_restriction_expression(expression->f_types, linenumber, filename, linebuf);
+            if (expression->fs_type != NULL) {
+#ifdef HAVE_FSTYPE
+                rs.fs_type = get_fs_type_from_string(expression->fs_type);
+                if (rs.fs_type == 0) {
+                    LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_ERROR, "invalid file system restriction '%s'", expression->fs_type)
+                        exit(INVALID_CONFIGURELINE_ERROR);
+                }
+                log_msg(eval_log_level, "eval(%p): file system restriction '%s' evaluates to 0x%lx", (void*) expression, expression->fs_type, rs.fs_type);
+#else
+                LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_ERROR, "%s", "file system type restriction is not supported on non-Linux platforms")
+                exit(INVALID_CONFIGURELINE_ERROR);
+#endif
+            }
+            log_msg(eval_log_level, "eval(%p): restriction {"
+#ifdef HAVE_FSTYPE
+                    "fs_type 0x%lx, "
+#endif
+                    "file type: %u)", (void*) expression,
+#ifdef HAVE_FSTYPE
+                    rs.fs_type,
+#endif
+                    rs.f_type);
+        }
+    } else {
+        log_msg(eval_log_level, "eval(%p): restriction is NULL, returning {"
+#ifdef HAVE_FSTYPE
+                    "fs_type 0x%lx, "
+#endif
+                    "file type: %u)", (void*) expression,
+#ifdef HAVE_FSTYPE
+                    rs.fs_type,
+#endif
+                    rs.f_type);
     }
     return rs;
 }
@@ -716,7 +815,7 @@ static void eval_rule_statement(rule_statement statement, char* rule_prefix, int
 void eval_config(ast* config_ast, int include_depth, char *rule_prefix) {
     ast* node = NULL;
     for(node = config_ast; node != NULL; node = node->next) {
-        log_msg(eval_log_level, "eval(%p): ast node (next: %p)", node, node->next);
+        log_msg(eval_log_level, "eval(%p): ast node (next: %p)", (void*) node, (void*) node->next);
         switch (node->type) {
             case config_option_type:
                 eval_config_statement(node->statement._config, node->linenumber, node->filename, node->linebuf);
