@@ -1,7 +1,7 @@
 /*
  * AIDE (Advanced Intrusion Detection Environment)
  *
- * Copyright (C) 1999-2006, 2010-2011, 2013, 2019-2023 Rami Lehti,
+ * Copyright (C) 1999-2006, 2010-2011, 2013, 2019-2025 Rami Lehti,
  *               Pablo Virolainen, Richard van den Berg, Hannes von Haugwitz
  *
  * This program is free software; you can redistribute it and/or
@@ -23,6 +23,7 @@
 #include <stdbool.h>
 #include <stdio.h>
 #include <string.h>
+#include <errno.h>
 #include <time.h>
 #include "attributes.h"
 #include "config.h"
@@ -31,7 +32,6 @@
 #include <stdlib.h>
 #include "db.h"
 #include "db_line.h"
-#include "db_lex.h"
 #include "db_file.h"
 #include "md.h"
 
@@ -46,8 +46,6 @@
 #include "base64.h"
 #include "util.h"
 
-db_line* db_char2line(char**, database*);
-
 static long readoct(char* s, database* db, char* field_name){
   long i;
   char* e;
@@ -68,6 +66,19 @@ static long readlong(char* s, database* db, char* field_name){
   return i;
 }
 
+#ifdef HAVE_FSTYPE
+static unsigned long readunsignedlong(char* s, database* db, char* field_name){
+  unsigned long i;
+  char* e;
+  i=strtoul(s,&e,10);
+  if (e[0]!='\0') {
+      LOG_DB_FORMAT_LINE(LOG_LEVEL_WARNING, "could not read '%s' from database: strtoul failed for '%s'", field_name, s)
+      return 0;
+  }
+  return i;
+}
+#endif
+
 static long long readlonglong(char* s, database* db, char* field_name){
   long long int i;
   char* e;
@@ -78,6 +89,35 @@ static long long readlonglong(char* s, database* db, char* field_name){
   return i;
 }
 
+static time_t read_time_t(char *str, database *db, const char *field_name) {
+    char *decoded = NULL;
+    char *time_t_str = str;
+    if (strcmp(str, "0") == 0) {
+        return 0;
+    }
+    if (*str >= 'M' && *str <= 'O') {
+        /* handle legacy base64 representation */
+        decoded = (char *) decode_base64(str, strlen(str), NULL);
+        if (decoded == NULL) {
+            /* warning is logged in decode_base64 */
+            return 0;
+        }
+        time_t_str = decoded;
+        log_msg(LOG_LEVEL_TRACE, "read_time_t: base64 decoded '%s' to '%s'", str, decoded);
+    }
+    char *endp = NULL;
+    time_t t = strtol(time_t_str, &endp, 10);
+    if (endp[0] != '\0' || (errno = ERANGE && (t == LONG_MAX || t == LONG_MIN))) {
+        LOG_DB_FORMAT_LINE(LOG_LEVEL_WARNING, "could not read '%s' from database: strtol failed for '%s' (setting '%s' to 0)", field_name, time_t_str, field_name)
+        t = 0;
+    } else {
+        log_msg(LOG_LEVEL_TRACE, "read_time_t: converted '%s' '%s' to %lld ", field_name, time_t_str, (long long)t);
+    }
+
+    free(decoded);
+    return t;
+}
+
 static struct md_container *init_db_attrs(url_t *u) {
     struct md_container *mdc = NULL;
     if (conf->db_attrs) {
@@ -92,7 +132,7 @@ static struct md_container *init_db_attrs(url_t *u) {
             case url_ftp: {
                 mdc = checked_malloc(sizeof(struct md_container)); /* freed in close_db_attrs */
                 mdc->todo_attr = conf->db_attrs;
-                init_md(mdc, u->value);
+                init_md(mdc, u->value, NULL);
 
                 break;
             }
@@ -110,12 +150,12 @@ static db_line *close_db_attrs (database *db) {
     db_line *line = NULL;
     if (db->mdc != NULL) {
         md_hashsums hs;
-        close_md(db->mdc, &hs, (db->url)->value);
         line = checked_malloc(sizeof(struct db_line));
         line->filename = (db->url)->value;
         line->perm = 0;
         line->attr = conf->db_attrs;
-        hashsums2line(&hs, line);
+        close_md(db->mdc, &hs, line->filename, NULL);
+        hashsums2line(&hs, line, NULL);
         free(db->mdc);
     }
     return line;
@@ -124,10 +164,12 @@ static db_line *close_db_attrs (database *db) {
 int db_init(database* db, bool readonly, bool gzip) {
   void* fp = NULL;
   
-  log_msg(LOG_LEVEL_TRACE,"db_init(): arguments: db=%p, gzip=%s", db, btoa(gzip));
+  log_msg(LOG_LEVEL_TRACE,"db_init(): arguments: db=%p, gzip=%s", (void*) db, btoa(gzip));
   
     db->mdc = init_db_attrs(db->url);
-    fp=be_init(readonly, db->url, gzip, false, db->linenumber, db->filename, db->linebuf);
+    bool created = false;
+    fp=be_init(db->url, readonly, gzip, false, db->linenumber, db->filename, db->linebuf, &created);
+    if (created) { db->flags |= DB_FLAG_CREATED; }
     if(fp==NULL) {
       return RETFAIL;
     } else {
@@ -144,27 +186,8 @@ int db_init(database* db, bool readonly, bool gzip) {
     }
 }
 
-db_line* db_readline(database* db){
-  db_line* s=NULL;
-
-  if (db->fp != NULL) {
-      char** ss=db_readline_file(db);
-      if (ss!=NULL){
-          s=db_char2line(ss,db);
-
-          for(int i=0;i<db->num_fields;i++){
-              if(db->fields[i]!=attr_unknown &&
-                      ss[db->fields[i]]!=NULL){
-                  free(ss[db->fields[i]]);
-                  ss[db->fields[i]]=NULL;
-              }
-          }
-          free(ss);
-      }
-  }
-  
-  return s;
-  
+db_entry_t db_readline(database* db, bool include_limited_entries){
+  return db_readline_file(db, include_limited_entries);
 }
 
 byte* base64tobyte(char* src,int len,size_t *ret_len)
@@ -175,7 +198,7 @@ byte* base64tobyte(char* src,int len,size_t *ret_len)
   return NULL;
 }
 
-static char *db_readchar(char *s)
+static char *read_linkname(char *s)
 {
   if (s == NULL)
     return (NULL);
@@ -188,12 +211,8 @@ static char *db_readchar(char *s)
     if (s[1] == '-')
       return (checked_strdup(""));
 
-    if (s[1] == '0')
-    {
-      memmove(s, s+1, strlen(s+1)+1);
-      // Hope this removes core
-      // dumping in some environments. Has something to do with
-      // memory (de)allocation.
+    if (s[1] == '0') {
+      s++;
     }
   }
 
@@ -207,6 +226,7 @@ static char *db_readchar(char *s)
 case attr_ ##hash : { \
     line->hashsums[hash_ ##hash]=base64tobyte(ss[db->fields[i]], \
         strlen(ss[db->fields[i]]), NULL); \
+    log_msg(LOG_LEVEL_TRACE, "%s: copy " #hash " hashsum (%s) to %p", line->filename, ss[db->fields[i]], (void*) line->hashsums[hash_ ##hash]); \
   break; \
 }
 
@@ -246,14 +266,14 @@ db_line* db_char2line(char** ss, database* db){
 
   for(int i=0;i<db->num_fields;i++){
 
-    log_msg(LOG_LEVEL_TRACE, "db_char2line(): %d[%d]: '%s' (%p)", db->lineno, i, ss[i], ss[i]);
+    log_msg(LOG_LEVEL_TRACE, "db_char2line(): %ld[%d]: '%s' (%p)", db->lineno, i, ss[i], (void*) ss[i]);
 
     switch (db->fields[i]) {
     case attr_filename : {
       if(ss[db->fields[i]]!=NULL){
-	decode_string(ss[db->fields[i]]);
-	line->fullpath=checked_strdup(ss[db->fields[i]]);
-	line->filename=line->fullpath;
+          decode_string(ss[db->fields[i]]);
+          line->fullpath=checked_strdup(ss[db->fields[i]]);
+          line->filename=line->fullpath;
       } else {
         log_msg(LOG_LEVEL_ERROR, "db_char2line(): error while reading database");
 	exit(EXIT_FAILURE);
@@ -261,11 +281,11 @@ db_line* db_char2line(char** ss, database* db){
       break;
     }
     case attr_linkname : {
-      line->linkname = db_readchar(ss[db->fields[i]]);
+      line->linkname = read_linkname(ss[db->fields[i]]);
       break;
     }
     case attr_mtime : {
-      line->mtime=base64totime_t(ss[db->fields[i]], db, "mtime");
+      line->mtime=read_time_t(ss[db->fields[i]], db, "mtime");
       break;
     }
     case attr_bcount : {
@@ -273,11 +293,11 @@ db_line* db_char2line(char** ss, database* db){
       break;
     }
     case attr_atime : {
-      line->atime=base64totime_t(ss[db->fields[i]], db, "atime");
+      line->atime=read_time_t(ss[db->fields[i]], db, "atime");
       break;
     }
     case attr_ctime : {
-      line->ctime=base64totime_t(ss[db->fields[i]], db, "ctime");
+      line->ctime=read_time_t(ss[db->fields[i]], db, "ctime");
       break;
     }
     case attr_inode : {
@@ -310,6 +330,9 @@ db_line* db_char2line(char** ss, database* db){
     CHAR2HASH(gostr3411_94)
     CHAR2HASH(stribog256)
     CHAR2HASH(stribog512)
+    CHAR2HASH(sha512_256)
+    CHAR2HASH(sha3_256)
+    CHAR2HASH(sha3_512)
     case attr_acl : {
 #ifdef WITH_POSIX_ACL
       char *tval = NULL;
@@ -355,7 +378,8 @@ db_line* db_char2line(char** ss, database* db){
             size_t vsz = 0;
             
             tval = strtok(NULL, ",");
-            line->xattrs->ents[num].key = db_readchar(checked_strdup(tval));
+            decode_string(tval);
+            line->xattrs->ents[num].key = checked_strdup(tval);
             tval = strtok(NULL, ",");
             val = base64tobyte(tval, strlen(tval), &vsz);
             line->xattrs->ents[num].val = val;
@@ -403,6 +427,12 @@ db_line* db_char2line(char** ss, database* db){
       line->capabilities = (char *)val;
       break;
     }
+    case attr_fs_type : {
+#ifdef HAVE_FSTYPE
+      line->fs_type = readunsignedlong(ss[db->fields[i]], db, attributes[attr_fs_type].db_name);
+#endif
+      break;
+    }
     case attr_bsize :
     case attr_sizeg :
     case attr_rdev :
@@ -429,36 +459,6 @@ db_line* db_char2line(char** ss, database* db){
   return line;
 }
 
-time_t base64totime_t(char* s, database* db, const char* field_name){
-  
-  if(strcmp(s,"0")==0){
-      return 0;
-  }
-  byte* b=decode_base64(s,strlen(s),NULL);
-  char* endp;
-  
-  if (b==NULL) {
-    
-    /* Should we print error here? */
-    
-    return 0;
-  } else {
-    time_t t = strtol((char *)b,&endp,10);
-    
-    if (endp[0]!='\0') {
-      LOG_DB_FORMAT_LINE(LOG_LEVEL_WARNING, "could not read '%s' from database: strtoll failed for '%s' (base64 encoded value: '%s')", field_name, b, s)
-      free(b);
-      return 0;
-    }
-    log_msg(LOG_LEVEL_DEBUG, "base64totime_t: converted '%s': '%s' to %lld (base64 encoded value '%s')", field_name, b, (long long) t, s);
-    free(b);
-    return t;
-  }
-  
-  
-}
-
-
 int db_writespec(db_config* dbconf)
 {
     if(
@@ -482,14 +482,14 @@ int db_writeline(db_line* line,db_config* dbconf){
        (dbconf->gzip_dbout && dbconf->database_out.gzp) ||
 #endif
        (dbconf->database_out.fp!=NULL)) {
-      if (db_writeline_file(line,dbconf,dbconf->database_out.url)==RETOK) {
+      if (db_writeline_file(line)==RETOK) {
 	return RETOK;
       }
     }
   return RETFAIL;
 }
 
-void db_close() {
+void db_close(void) {
   if (conf->database_out.url) {
   switch (conf->database_out.url->type) {
   case url_stdin:
@@ -513,6 +513,7 @@ void db_close() {
 #ifdef WITH_CURL
         if (conf->database_out.fp!=NULL) {
             url_fclose(conf->database_out.fp);
+            conf->database_out.fp = NULL;
         }
 #endif /* WITH CURL */
       break;
