@@ -1,7 +1,7 @@
 /*
  * AIDE (Advanced Intrusion Detection Environment)
  *
- * Copyright (C) 2020,2022,2023 Hannes von Haugwitz
+ * Copyright (C) 2020,2022,2023,2025 Hannes von Haugwitz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,12 +24,11 @@
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
-#ifdef WITH_PTHREAD
 #include <pthread.h>
-#endif
 
 #include "log.h"
 #include "errorcodes.h"
+#include "util.h"
 
 LOG_LEVEL prev_log_level = LOG_LEVEL_UNSET;
 LOG_LEVEL log_level = LOG_LEVEL_UNSET;
@@ -42,37 +41,40 @@ typedef struct log_cache {
 log_cache *cached_lines = NULL;
 int ncachedlines = 0;
 
-#ifdef WITH_PTHREAD
-pthread_mutex_t log_mutex = PTHREAD_MUTEX_INITIALIZER;
+int colored_log = -1;
 
-void log_init() {
-    pthread_mutexattr_t mutex_attrs;
-    pthread_mutexattr_init(&mutex_attrs);
-    pthread_mutexattr_settype(&mutex_attrs, PTHREAD_MUTEX_RECURSIVE);
-    pthread_mutex_init(&log_mutex, &mutex_attrs);
-}
-#endif
+pthread_mutex_t log_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 struct log_level {
     LOG_LEVEL log_level;
     const char *name;
     const char *log_string;
+    const char *colored_log_string;
 };
 
 static struct log_level log_level_array[] = {
-    { LOG_LEVEL_ERROR,         "error",         "  ERROR" },
-    { LOG_LEVEL_WARNING,       "warning",       "WARNING" },
-    { LOG_LEVEL_NOTICE,        "notice",        " NOTICE" },
-    { LOG_LEVEL_INFO,          "info",          "   INFO" },
-    { LOG_LEVEL_RULE,          "rule",          "   RULE" },
-    { LOG_LEVEL_COMPARE,       "compare",       "COMPARE" },
-    { LOG_LEVEL_CONFIG,        "config",        " CONFIG" },
-    { LOG_LEVEL_DEBUG,         "debug",         "  DEBUG" },
-    { LOG_LEVEL_THREAD,        "thread",        " THREAD" },
-    { LOG_LEVEL_TRACE,         "trace",         "  TRACE" },
-    { 0,                       NULL,            NULL      },
+    { LOG_LEVEL_ERROR,         "error",         "  ERROR", COLOR_B_RED    "  ERROR" COLOR_RESET },
+    { LOG_LEVEL_WARNING,       "warning",       "WARNING", COLOR_B_YELLOW "WARNING" COLOR_RESET },
+    { LOG_LEVEL_NOTICE,        "notice",        " NOTICE", COLOR_L_ORANGE " NOTICE" COLOR_RESET },
+    { LOG_LEVEL_INFO,          "info",          "   INFO", COLOR_L_GREEN  "   INFO" COLOR_RESET },
+    { LOG_LEVEL_COMPARE,       "compare",       "COMPARE", COLOR_B_BLUE   "COMPARE" COLOR_RESET },
+    { LOG_LEVEL_RULE,          "rule",          "   RULE", COLOR_L_BLUE   "   RULE" COLOR_RESET },
+    { LOG_LEVEL_CONFIG,        "config",        " CONFIG", COLOR_L_CYAN   " CONFIG" COLOR_RESET },
+    { LOG_LEVEL_DEBUG,         "debug",         "  DEBUG", COLOR_B_PURPLE "  DEBUG" COLOR_RESET },
+    { LOG_LEVEL_LIMIT,         "limit",         "  LIMIT", COLOR_L_GRAY   "  LIMIT" COLOR_RESET },
+    { LOG_LEVEL_THREAD,        "thread",        " THREAD", COLOR_B_CYAN   " THREAD" COLOR_RESET },
+    { LOG_LEVEL_TRACE,         "trace",         "  TRACE", COLOR_L_PURPLE "  TRACE" COLOR_RESET },
+    { 0,                       NULL,            NULL     , NULL      },
 };
 
+static const char* get_log_string(LOG_LEVEL level) {
+    if (colored_log) {
+        return log_level_array[level-1].colored_log_string;
+    } else {
+        return log_level_array[level-1].log_string;
+    }
+}
+
 static void cache_line(LOG_LEVEL, const char*, va_list)
 #ifdef __GNUC__
     __attribute__ ((format (printf, 2, 0)))
@@ -81,11 +83,9 @@ static void cache_line(LOG_LEVEL, const char*, va_list)
 static void cache_line(LOG_LEVEL level, const char* format, va_list ap) {
     int n;
 
-    FILE* url = stderr;
-
     cached_lines = realloc(cached_lines, (ncachedlines+1) * sizeof(log_cache)); /* freed in log_cached_lines() */
     if (cached_lines == NULL) {
-        fprintf(url, "%s: realloc: failed to allocate memory\n", log_level_array[LOG_LEVEL_ERROR-1].log_string);
+        stderr_msg("%s: realloc: failed to allocate memory\n", get_log_string(LOG_LEVEL_ERROR));
         exit(MEMORY_ALLOCATION_FAILURE);
     }
 
@@ -100,7 +100,7 @@ static void cache_line(LOG_LEVEL level, const char* format, va_list ap) {
     int size = n * sizeof(char);
     cached_lines[ncachedlines].message = malloc(size); /* freed in log_cached_lines() */
     if (cached_lines[ncachedlines].message == NULL) {
-        fprintf(url, "%s: malloc: failed to allocate %d bytes of memory\n", log_level_array[LOG_LEVEL_ERROR-1].log_string, size);
+        stderr_msg("%s: malloc: failed to allocate %d bytes of memory\n", get_log_string(LOG_LEVEL_ERROR), size);
         exit(MEMORY_ALLOCATION_FAILURE);
     }
 
@@ -113,17 +113,17 @@ const char * get_log_level_name(LOG_LEVEL level) {
 }
 
 static void log_cached_lines(void) {
-    FILE* url = stderr;
-
+    pthread_mutex_lock(&log_mutex);
     for(int i = 0; i < ncachedlines; ++i) {
         LOG_LEVEL level = cached_lines[i].level;
         if (level == LOG_LEVEL_ERROR || level <= log_level) {
-            fprintf(url, "%s: %s\n", log_level_array[level-1].log_string, cached_lines[i].message);
+            stderr_msg("%s: %s\n", get_log_string(level), cached_lines[i].message);
         }
         free(cached_lines[i].message);
     }
     ncachedlines = 0;
     free(cached_lines);
+    pthread_mutex_unlock(&log_mutex);
 }
 
 static void vlog_msg(LOG_LEVEL, const char*, va_list)
@@ -132,18 +132,16 @@ static void vlog_msg(LOG_LEVEL, const char*, va_list)
 #endif
 ;
 static void vlog_msg(LOG_LEVEL level,const char* format, va_list ap) {
-    FILE* url = stderr;
-
-    if (level == LOG_LEVEL_ERROR || level <= log_level) {
-        fprintf(url, "%s: ", log_level_array[level-1].log_string );
-        vfprintf(url, format, ap);
-        fprintf(url, "\n");
-    } else if (log_level == LOG_LEVEL_UNSET) {
+    if (level != LOG_LEVEL_ERROR && (log_level == LOG_LEVEL_UNSET || colored_log < 0)) {
+    pthread_mutex_lock(&log_mutex);
         cache_line(level, format, ap);
+    pthread_mutex_unlock(&log_mutex);
+    } else if (level == LOG_LEVEL_ERROR || level <= log_level) {
+        vstderr_prefix_line(get_log_string(level), format, ap);
     }
 }
 
-bool is_log_level_unset() {
+bool is_log_level_unset(void) {
     return log_level == LOG_LEVEL_UNSET;
 }
 
@@ -155,13 +153,19 @@ LOG_LEVEL get_log_level_from_string(char* val) {
             return level->log_level;
         }
     }
-
     return LOG_LEVEL_UNSET;
 }
 
+void set_colored_log(bool color) {
+    colored_log = color;
+    if (ncachedlines && log_level != LOG_LEVEL_UNSET) {
+        log_cached_lines();
+    }
+}
+
 void set_log_level(LOG_LEVEL level) {
     log_level = level;
-    if (ncachedlines && level != LOG_LEVEL_UNSET) {
+    if (colored_log >= 0 && ncachedlines && log_level != LOG_LEVEL_UNSET) {
         log_cached_lines();
     }
 }
@@ -181,38 +185,9 @@ LOG_LEVEL toogle_log_level(LOG_LEVEL level) {
     return log_level;
 }
 
-void log_msg(LOG_LEVEL, const char*, ...)
-#ifdef __GNUC__
-    __attribute__ ((format (printf, 2, 3)))
-#endif
-;
 void log_msg(LOG_LEVEL level, const char* format, ...) {
-#ifdef WITH_PTHREAD
-    pthread_mutex_lock(&log_mutex);
-#endif
     va_list argp;
     va_start(argp, format);
     vlog_msg(level, format, argp);
     va_end(argp);
-#ifdef WITH_PTHREAD
-    pthread_mutex_unlock(&log_mutex);
-#endif
-}
-
-void stderr_msg(const char* format, ...)
-#ifdef __GNUC__
-    __attribute__ ((format (printf, 1, 2)))
-#endif
-;
-void stderr_msg(const char* format, ...) {
-#ifdef WITH_PTHREAD
-    pthread_mutex_lock(&log_mutex);
-#endif
-    va_list ap;
-    va_start(ap, format);
-    vfprintf(stderr, format, ap);
-    va_end(ap);
-#ifdef WITH_PTHREAD
-    pthread_mutex_unlock(&log_mutex);
-#endif
 }
