@@ -1,7 +1,7 @@
 /*
  * AIDE (Advanced Intrusion Detection Environment)
  *
- * Copyright (C) 1999-2007, 2010-2013, 2015-2016, 2018-2023 Rami Lehti,
+ * Copyright (C) 1999-2007, 2010-2013, 2015-2016, 2018-2025 Rami Lehti,
  *               Pablo Virolainen, Mike Markley, Richard van den Berg,
  *               Hannes von Haugwitz
  *
@@ -24,7 +24,6 @@
 #include "aide.h"
 #include <stdlib.h>
 #include <string.h>
-#include <time.h>
 #include <math.h>
 #ifdef WITH_AUDIT
 #include <libaudit.h>
@@ -44,7 +43,9 @@
 #include "log.h"
 #include "rx_rule.h"
 #include "seltree_struct.h"
-
+#ifdef HAVE_FSTYPE
+#include "file.h"
+#endif
 #include "attributes.h"
 #include "base64.h"
 #include "conf_ast.h"
@@ -101,6 +102,9 @@ const ATTRIBUTE report_attrs_order[] = {
 #ifdef WITH_CAPABILITIES
    attr_capabilities,
 #endif
+#ifdef HAVE_FSTYPE
+    attr_fs_type,
+#endif
 };
 
 int report_attrs_order_length = sizeof(report_attrs_order)/sizeof(ATTRIBUTE);
@@ -131,7 +135,7 @@ static struct report_level report_level_array[] = {
  { REPORT_LEVEL_CHANGED_ATTRIBUTES, "changed_attributes" },
  { REPORT_LEVEL_ADDED_REMOVED_ATTRIBUTES, "added_removed_attributes" },
  { REPORT_LEVEL_ADDED_REMOVED_ENTRIES, "added_removed_entries" },
- { 0, NULL }
+ { REPORT_LEVEL_UNKNOWN, NULL }
 };
 
 struct report_format {
@@ -142,7 +146,7 @@ struct report_format {
 static struct report_format report_format_array[] = {
  { REPORT_FORMAT_PLAIN, "plain" },
  { REPORT_FORMAT_JSON, "json" },
- { 0, NULL }
+ { REPORT_FORMAT_UNKNOWN, NULL }
 };
 
 #ifdef WITH_XATTR
@@ -198,6 +202,7 @@ static int xattrs2array(xattrs_type* xattrs, char* **values) {
 #ifdef WITH_ACL
 static int acl2array(acl_type* acl, char* **values) {
     int n = 0;
+    *values = NULL;
 #ifdef WITH_POSIX_ACL
 #define easy_posix_acl(x,y) \
         if (acl->x) { \
@@ -245,7 +250,6 @@ char* get_file_type_string(mode_t mode) {
 #ifdef S_IFPORT
         case S_IFPORT: return _("Port");
 #endif
-        case 0: return NULL;
         default: return _("Unknown file type");
     }
 }
@@ -266,7 +270,7 @@ REPORT_LEVEL get_report_level(char *str) {
             return level->report_level;
         }
     }
-    return 0;
+    return REPORT_LEVEL_UNKNOWN;
 }
 
 static const char* get_report_format_string(REPORT_FORMAT report_format) {
@@ -281,7 +285,7 @@ REPORT_FORMAT get_report_format(char *str) {
             return level->report_format;
         }
     }
-    return 0;
+    return REPORT_FORMAT_UNKNOWN;
 }
 
 static void report_vprintf(report_t*, const char *, va_list)
@@ -320,11 +324,6 @@ if (!r->quiet || (r->nadd || r->nchg || r->nrem)) {
 
 }
 
-void report_printf(report_t*, const char*, ...)
-#ifdef __GNUC__
-        __attribute__ ((format (printf, 2, 3)))
-#endif
-;
 void report_printf(report_t* r, const char* error_msg, ...) {
     va_list ap;
 
@@ -347,7 +346,7 @@ void log_report_urls(LOG_LEVEL log_level) {
     for (l=conf->report_urls; l; l=l->next) {
         report_t* r = l->data;
 
-        log_msg(log_level, " %s%s%s (%p)", get_url_type_string((r->url)->type), (r->url)->value?":":"", (r->url)->value?(r->url)->value:"", r);
+        log_msg(log_level, " %s%s%s (%p)", get_url_type_string((r->url)->type), (r->url)->value?":":"", (r->url)->value?(r->url)->value:"", (void*) r);
 
         log_msg(log_level, "   level: %s | format: %s | base16: %s | append: %s | quiet: %s | detailed_init: %s | summarize_changes: %s | grouped: %s", get_report_level_string(r->level), get_report_format_string(r->format), btoa(r->base16), btoa(r->append), btoa(r->quiet), btoa(r->detailed_init), btoa(r->summarize_changes), btoa(r->grouped));
         char *str;
@@ -480,13 +479,14 @@ bool add_report_url(url_t* url, int linenumber, char* filename, char* linebuf) {
     r->ignore_e2fsattrs = conf->report_ignore_e2fsattrs;
 #endif
 
-    log_msg(LOG_LEVEL_DEBUG, _("add report_url (%p): url(: %s:%s, level: %d"), r, get_url_type_string((r->url)->type), (r->url)->value, r->level);
+    log_msg(LOG_LEVEL_DEBUG, _("add report_url (%p): url(: %s, level: %d"), (void*) r, (r->url)->raw, r->level);
     conf->report_urls=list_sorted_insert(conf->report_urls, (void*) r, compare_report_t_by_report_level);
+    LOG_CONFIG_FORMAT_LINE(LOG_LEVEL_CONFIG, "set 'report_url' to '%s'", url->raw)
     return true;
 
 }
 
-bool init_report_urls() {
+bool init_report_urls(void) {
     list* l = NULL;
 
     for (l=conf->report_urls; l; l=l->next) {
@@ -501,7 +501,7 @@ bool init_report_urls() {
         }
 #endif
         default : {
-            r->fd=be_init(0,r->url,0, r->append, r->linenumber, r->filename, r->linebuf);
+            r->fd=be_init(r->url, false, false, r->append, r->linenumber, r->filename, r->linebuf, NULL);
             if(r->fd==NULL) {
                 return false;
             }
@@ -512,27 +512,32 @@ bool init_report_urls() {
     }
     return true;
 }
-static char* byte_to_base16(byte* src, size_t ssize) {
-    char* str = checked_malloc((2*ssize+1) * sizeof (char));
-    size_t i;
-    for(i=0; i < ssize; ++i) {
-        snprintf(&str[2*i], 3, "%02x", src[i]);
+
+static int attributes2array(DB_ATTR_TYPE attrs, char* **values) {
+    *values = NULL;
+
+    int n = 0;
+    for (ATTRIBUTE i = 0; i < num_attrs; ++i) {
+        if (attributes[i].db_name && (1LLU<<i)&attrs) {
+            n++;
+        }
     }
-    return str;
-}
 
-char* get_time_string(const time_t *tm) {
-    const char time_format[] = "%Y-%m-%d %H:%M:%S %z";
-    int time_string_len = strlen("1979-01-01 01:00:00 +0100")+1;
-
-    char *time = checked_malloc(time_string_len * sizeof (char));
-    strftime(time, time_string_len, time_format, localtime(tm));
-
-    return time;
+    *values = checked_malloc(n * sizeof(char*));
+    n = 0;
+    for (ATTRIBUTE i = 0; i < num_attrs; ++i) {
+        if (attributes[i].db_name && (1LLU<<i)&attrs) {
+            size_t len = strlen(attributes[i].db_name)+1;
+            (*values)[n] = checked_malloc(len * sizeof(char));
+            snprintf((*values)[n], len, "%s", attributes[i].db_name);
+            n++;
+        }
+    }
+    return n;
 }
 
 int get_attribute_values(DB_ATTR_TYPE attr, db_line* line,
-        char* **values, report_t* r) {
+        char* **values, report_t *r) {
 
 #define easy_string(s) \
 l = strlen(s)+1; \
@@ -549,6 +554,27 @@ snprintf(*values[0], l, "%s",s);
 } else if (a&attr) { \
     *values[0] = get_time_string(&(line->b)); \
 
+    if (line!=NULL && attr&get_hashes(true)) {
+        for (int i = 0 ; i < num_hashes ; ++i) {
+            if (ATTR(hashsums[i].attribute)&attr) {
+                if (line->hashsums[i]) {
+                    *values = checked_malloc(1 * sizeof (char*));
+                    if (r==NULL || r->base16) {
+                        *values[0] = byte_to_base16(line->hashsums[i], hashsums[i].length);
+                    } else {
+                        *values[0] = encode_base64(line->hashsums[i], hashsums[i].length);
+                    }
+                    return 1;
+                } else {
+                    return 0;
+                }
+            }
+        }
+    }
+
+    if (ATTR(attr_attr)&attr) {
+        return attributes2array(line->attr, values);
+    } else
     if (line==NULL || !(line->attr&attr)) {
         *values = NULL;
         return 0;
@@ -565,6 +591,10 @@ snprintf(*values[0], l, "%s",s);
         *values = checked_malloc(1 * sizeof (char*));
         if (ATTR(attr_ftype)&attr) {
             easy_string(get_file_type_string(line->perm))
+#ifdef HAVE_FSTYPE
+        } else if (ATTR(attr_fs_type)&attr) {
+            *values[0] = get_fs_type_string_from_magic(line->fs_type);
+#endif
         } else if (ATTR(attr_linkname)&attr) {
             easy_string(line->linkname)
         easy_number((ATTR(attr_size)|ATTR(attr_sizeg)),size,"%lli")
@@ -584,25 +614,13 @@ snprintf(*values[0], l, "%s",s);
 #endif
 #ifdef WITH_E2FSATTRS
         } else if (ATTR(attr_e2fsattrs)&attr) {
-            *values[0]=get_e2fsattrs_string(line->e2fsattrs, false, r->ignore_e2fsattrs);
+            *values[0]=get_e2fsattrs_string(line->e2fsattrs, false, r?r->ignore_e2fsattrs:0L);
 #endif
 #ifdef WITH_CAPABILITIES
         } else if (ATTR(attr_capabilities)&attr) {
             easy_string(line->capabilities)
 #endif
         } else {
-
-  for (int i = 0 ; i < num_hashes ; ++i) {
-    if (ATTR(hashsums[i].attribute)&attr) {
-        if (r->base16) {
-            *values[0] = byte_to_base16(line->hashsums[i], hashsums[i].length);
-        } else {
-            *values[0] = encode_base64(line->hashsums[i], hashsums[i].length);
-        }
-        return 1;
-      }
-  }
-
             easy_string("unknown attribute")
         }
         return 1;
@@ -613,7 +631,7 @@ char* get_summarize_changes_string(report_t* report, seltree* node) {
         int i;
         char* summary = checked_malloc ((report_attrs_order_length+1) * sizeof (char));
         if (node->checked&(NODE_ADDED|NODE_REMOVED)) {
-            summary[0]=get_file_type_char_from_perm(((node->checked&NODE_REMOVED)?node->old_data:node->new_data)->perm);
+            summary[0]=get_f_type_char_from_perm(((node->checked&NODE_REMOVED)?node->old_data:node->new_data)->perm);
             for(i=1;i<report_attrs_order_length;i++){
                 summary[i]=(node->checked&NODE_ADDED)?'+':'-';
             }
@@ -625,7 +643,7 @@ char* get_summarize_changes_string(report_t* report, seltree* node) {
                 d = '-'; a = '+'; g = ':'; u = '.'; s = ' ';
                 switch (i) {
                     case 0:
-                        summary[i]=get_file_type_char_from_perm((node->new_data)->perm);
+                        summary[i]=get_f_type_char_from_perm((node->new_data)->perm);
                         continue;
                     case 2:
                         if (attrs&(node->changed_attrs&(~(report->ignore_removed_attrs))) && (node->old_data)->size > (node->new_data)->size) {
@@ -685,9 +703,9 @@ static DB_ATTR_TYPE get_report_attributes(seltree* node, report_t *report) {
 }
 
 static void terse_report(seltree* node) {
-    list* n = NULL;
     list* l = NULL;
 
+    pthread_rwlock_rdlock(&node->rwlock);
     for (l=conf->report_urls; l; l=l->next) {
         report_t* r = l->data;
 
@@ -756,27 +774,30 @@ static void terse_report(seltree* node) {
         removed_entries_reported |= r->nrem != 0;
         changed_entries_reported |= r->nchg != 0;
     }
-    for (n=node->childs;n;n=n->next) {
-        terse_report((seltree*)n->data);
+
+    for(tree_node *x = tree_walk_first(node->children); x != NULL ; x = tree_walk_next(x)) {
+        terse_report(tree_get_data(x));
     }
+    pthread_rwlock_unlock(&node->rwlock);
 }
 
-void print_report_entries(report_t *report, seltree* node, const int node_status, void (*print_line)(report_t*, seltree*)) {
-    list* r=NULL;
+void print_report_entries(report_t *report, seltree* node, const int node_status, void (*print_line)(report_t*, char*, int, seltree*)) {
 
+    pthread_rwlock_rdlock(&node->rwlock);
     if (node->checked&node_status)  {
             if (!(node->changed_attrs) || ~(report->ignore_changed_attrs)&(node->changed_attrs
 #ifdef WITH_E2FSATTRS
                 & (~ATTR(attr_e2fsattrs) | (node->changed_attrs&ATTR(attr_e2fsattrs) && ~(report->ignore_e2fsattrs)&(node->old_data->e2fsattrs^node->new_data->e2fsattrs)?ATTR(attr_e2fsattrs):0))
 #endif
             )) {
-        print_line(report, node);
+        print_line(report, ((node->checked&NODE_REMOVED)?node->old_data:node->new_data)->filename, node->checked, node);
             }
 
     }
-    for(r=node->childs;r;r=r->next){
-        print_report_entries(report, (seltree*)r->data, node_status, print_line);
+    for(tree_node *x = tree_walk_first(node->children); x != NULL ; x = tree_walk_next(x)) {
+        print_report_entries(report, tree_get_data(x), node_status, print_line);
     }
+    pthread_rwlock_unlock(&node->rwlock);
 }
 
 void print_dbline_attrs(report_t * report, db_line* oline, db_line* nline, DB_ATTR_TYPE report_attrs, void (*print_attribute)(report_t *, db_line*, db_line*, ATTRIBUTE)) {
@@ -812,7 +833,7 @@ void print_databases_attrs(report_t *report, void (*print_database_attributes)(r
 }
 
 void print_report_details(report_t *report, seltree* node, void (*print_attributes)(report_t *, db_line*, db_line*, DB_ATTR_TYPE)) {
-    list* r=NULL;
+    pthread_rwlock_rdlock(&node->rwlock);
     if (node->checked&NODE_CHANGED) {
         print_attributes(report, node->old_data, node->new_data, get_report_attributes(node, report));
     }
@@ -824,14 +845,15 @@ void print_report_details(report_t *report, seltree* node, void (*print_attribut
             print_attributes(report, node->old_data, NULL, (node->old_data)->attr&~(report->ignore_removed_attrs));
         }
     }
-    for(r=node->childs;r;r=r->next){
-        print_report_details(report, (seltree*)r->data, print_attributes);
+    for(tree_node *x = tree_walk_first(node->children); x != NULL ; x = tree_walk_next(x)) {
+        print_report_details(report, tree_get_data(x), print_attributes);
     }
+    pthread_rwlock_unlock(&node->rwlock);
 }
 
 #ifdef WITH_AUDIT
   /* Something changed, send audit anomaly message */
-void send_audit_report()
+void send_audit_report(void)
 {
   if(nadd!=0||nrem!=0||nchg!=0){
     int fd=audit_open();
@@ -864,7 +886,7 @@ static void print_report(report_t * report, seltree * node, report_format_module
 
     if (report->level >= REPORT_LEVEL_SUMMARY) {
         char *time = get_time_string(&(conf->start_time));
-        module.print_report_starttime_version(report, time, AIDEVERSION);
+        module.print_report_starttime_version(report, time, conf->aide_version);
         free(time); time=NULL;
     }
 
@@ -941,9 +963,12 @@ int gen_report(seltree* node) {
             case REPORT_FORMAT_PLAIN:
                 print_report(report, node, report_module_plain);
                 break;
-            case REPORT_FORMAT_JSON: ;
+            case REPORT_FORMAT_JSON:
                 print_report(report, node, report_module_json);
                 break;
+            case REPORT_FORMAT_UNKNOWN:
+                /* skip unknown report format */
+                break;
         }
     }
 
