@@ -1,7 +1,7 @@
 /*
  * AIDE (Advanced Intrusion Detection Environment)
  *
- * Copyright (C) 2022 Hannes von Haugwitz
+ * Copyright (C) 2022-2025 Hannes von Haugwitz
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -27,7 +27,7 @@
 #include "db_config.h"
 #include "db_line.h"
 #include "report.h"
-#include "seltree_struct.h"
+#include "seltree.h"
 #include "stdbool.h"
 #include "url.h"
 
@@ -57,12 +57,48 @@ static int _escape_json_string(const char *src, char *escaped_string) {
     int n = 0;
 
     for (i = 0; i < strlen(src); ++i) {
-        if (src[i] == '\\') {
-            if (escaped_string) { escaped_string[n] = '\\'; }
-            n++;
+        switch(src[i]) {
+            case '\n':
+                if (escaped_string) { escaped_string[n] = '\\'; }
+                n++;
+                if (escaped_string) { escaped_string[n] = 'n'; }
+                n++;
+                break;
+            case '\t':
+                if (escaped_string) { escaped_string[n] = '\\'; }
+                n++;
+                if (escaped_string) { escaped_string[n] = 't'; }
+                n++;
+                break;
+            case '\b':
+                if (escaped_string) { escaped_string[n] = '\\'; }
+                n++;
+                if (escaped_string) { escaped_string[n] = 'b'; }
+                n++;
+                break;
+            case '\f':
+                if (escaped_string) { escaped_string[n] = '\\'; }
+                n++;
+                if (escaped_string) { escaped_string[n] = 'f'; }
+                n++;
+                break;
+            case '\r':
+                if (escaped_string) { escaped_string[n] = '\\'; }
+                n++;
+                if (escaped_string) { escaped_string[n] = 'r'; }
+                n++;
+                break;
+            case '"':
+            case '\\':
+                if (escaped_string) { escaped_string[n] = '\\'; }
+                n++;
+                if (escaped_string) { escaped_string[n] = src[i]; }
+                n++;
+                break;
+            default:
+                if (escaped_string) { escaped_string[n] = src[i]; }
+                n++;
         }
-        if (escaped_string) { escaped_string[n] = src[i]; }
-        n++;
     }
     if (escaped_string) { escaped_string[n] = '\0'; }
     n++;
@@ -99,30 +135,30 @@ static char* _get_value_format(ATTRIBUTE attribute) {
     }
 }
 
-static void _print_line(report_t* report, seltree* node) {
+static void print_line_json(report_t* report, char* filename, int node_checked, seltree* node) {
     if (line_first) { line_first=false; }
     else { report_printf(report,",\n"); }
 
-    char *escacped_filename = _get_escaped_json_string(((node->checked&NODE_REMOVED)?node->old_data:node->new_data)->filename);
+    char *escaped_filename = _get_escaped_json_string(filename);
 
     if(report->summarize_changes) {
         char* summary = get_summarize_changes_string(report, node);
-        report_printf(report, JSON_FMT_STRING_PLAIN, 4, ' ', escacped_filename, summary);
+        report_printf(report, JSON_FMT_STRING_PLAIN, 4, ' ', escaped_filename, summary);
         free(summary); summary=NULL;
     } else if (!report->grouped) {
         char* change_type;
-        if (node->checked&NODE_ADDED) {
+        if (node_checked&NODE_ADDED) {
             change_type = "added";
-        } else if (node->checked&NODE_REMOVED) {
+        } else if (node_checked&NODE_REMOVED) {
             change_type = "removed";
         } else {
             change_type = "changed";
         }
-        report_printf(report, JSON_FMT_STRING_PLAIN, 4, ' ', escacped_filename, change_type);
+        report_printf(report, JSON_FMT_STRING_PLAIN, 4, ' ', escaped_filename, change_type);
     } else {
-        report_printf(report, JSON_FMT_ARRAY_ELEMENT_PLAIN, 4, ' ', escacped_filename);
+        report_printf(report, JSON_FMT_ARRAY_ELEMENT_PLAIN, 4, ' ', escaped_filename);
     }
-    free(escacped_filename);
+    free(escaped_filename);
 }
 
 static void _print_attribute_value(report_t *report, const char* name, ATTRIBUTE attribute, char **value, int num, int ident) {
@@ -192,14 +228,14 @@ static void _print_database_attributes(report_t *report, db_line* db) {
     if (databases_first) { databases_first=false; }
     else { report_printf(report,",\n"); }
 
-    char *escacped_filename = _get_escaped_json_string(db->filename);
+    char *escaped_filename = _get_escaped_json_string(db->filename);
 
     database_first = true;
-    report_printf(report, JSON_FMT_OBJECT_BEGIN, 4, ' ', escacped_filename);
+    report_printf(report, JSON_FMT_OBJECT_BEGIN, 4, ' ', escaped_filename);
     print_dbline_attrs(report, db, NULL, db->attr, _print_database_attribute);
     report_printf(report,"\n");
     report_printf(report, JSON_FMT_OBJECT_END_PLAIN, 4, ' ');
-    free(escacped_filename);
+    free(escaped_filename);
 }
 
 static void _print_report_dbline_attributes(report_t *report, db_line* oline, db_line* nline, DB_ATTR_TYPE report_attrs) {
@@ -207,13 +243,13 @@ static void _print_report_dbline_attributes(report_t *report, db_line* oline, db
         if (details_first) { details_first=false; }
         else { report_printf(report,",\n"); }
 
-        char *escacped_filename = _get_escaped_json_string((nline==NULL?oline:nline)->filename);
-        report_printf(report, JSON_FMT_OBJECT_BEGIN, 4, ' ', escacped_filename);
+        char *escaped_filename = _get_escaped_json_string((nline==NULL?oline:nline)->filename);
+        report_printf(report, JSON_FMT_OBJECT_BEGIN, 4, ' ', escaped_filename);
         attributes_first = true;
         print_dbline_attrs(report, oline, nline, report_attrs, _print_attribute);
         report_printf(report,"\n");
         report_printf(report, JSON_FMT_OBJECT_END_PLAIN, 4, ' ');
-        free(escacped_filename);
+        free(escaped_filename);
     }
 }
 
@@ -292,7 +328,7 @@ static void print_report_entries_json(report_t *report, seltree* node, const int
             report_printf(report, report->summarize_changes||!report->grouped?JSON_FMT_OBJECT_BEGIN:JSON_FMT_ARRAY_BEGIN, 2, ' ', "entries");
             break;
     }
-    print_report_entries(report, node, filter, _print_line);
+    print_report_entries(report, node, filter, print_line_json);
     report_printf(report,"\n");
     report_printf(report, report->summarize_changes||!report->grouped?JSON_FMT_OBJECT_END:JSON_FMT_ARRAY_END, 2, ' ');
 }
@@ -301,11 +337,12 @@ static void print_report_diff_attrs_entries_json(report_t *report) {
     if (report->num_diff_attrs_entries) {
         report_printf(report, JSON_FMT_OBJECT_BEGIN, 2, ' ', "different_attributes");
         for(int i = 0; i < report->num_diff_attrs_entries; ++i) {
-            char *str = NULL;
-            report_printf(report, i+1<report->num_diff_attrs_entries?JSON_FMT_STRING_COMMA:JSON_FMT_STRING_LAST , 4, ' ',
-                    report->diff_attrs_entries[i].entry,
-                    str= diff_attributes(report->diff_attrs_entries[i].old_attrs, report->diff_attrs_entries[i].new_attrs));
-            free(str);
+            char *escaped_filename = _get_escaped_json_string(report->diff_attrs_entries[i].entry);
+            char *attrs = diff_attributes(report->diff_attrs_entries[i].old_attrs, report->diff_attrs_entries[i].new_attrs);
+            report_printf(report, i + 1 < report->num_diff_attrs_entries ? JSON_FMT_STRING_COMMA : JSON_FMT_STRING_LAST, 4, ' ',
+                    escaped_filename, attrs);
+            free(escaped_filename);
+            free(attrs);
         }
         report->num_diff_attrs_entries = 0;
         free(report->diff_attrs_entries);
