@@ -1,7 +1,7 @@
 /*
  * AIDE (Advanced Intrusion Detection Environment)
  *
- * Copyright (C) 1999-2006, 2010, 2011, 2013, 2015-2016, 2018-2022 Rami Lehti,
+ * Copyright (C) 1999-2006, 2010, 2011, 2013, 2015-2016, 2018-2025 Rami Lehti,
  *               Pablo Virolainen, Mike Markley, Richard van den Berg,
  *               Hannes von Haugwitz
  *
@@ -29,13 +29,11 @@
 #include "db_config.h"
 #include "db_line.h"
 #include "report.h"
-#include "seltree_struct.h"
+#include "seltree.h"
 #include "stdbool.h"
 #include "url.h"
 #include "locale-aide.h"
 
-const int width_details = 80;
-
 #define PLAIN_REPORT_HEADLINE_FMT "\n\n---------------------------------------------------\n%s:\n---------------------------------------------------\n"
 
 bool first = true;
@@ -66,7 +64,7 @@ static void _print_attribute(report_t *report, db_line* oline, db_line* nline, A
     char **ovalue = NULL;
     char **nvalue = NULL;
     int onumber, nnumber, i, c;
-    int p = (width_details-(4 + MAX_WIDTH_DETAILS_STRING))/2;
+    int p = (conf->print_details_width-(4 + MAX_WIDTH_DETAILS_STRING))/2;
 
     DB_ATTR_TYPE attr = ATTR(attribute);
     const char* name = attributes[attribute].details_string;
@@ -135,30 +133,30 @@ static void print_report_summary_plain(report_t *report) {
     }
 }
 
-static void print_line_plain(report_t* report, seltree* node) {
+static void print_line_plain(report_t* report, char* filename, int node_checked, seltree* node) {
     if(report->summarize_changes) {
         char* summary = get_summarize_changes_string(report, node);
-        report_printf(report, "\n%s: %s", summary, ((node->checked&NODE_REMOVED)?node->old_data:node->new_data)->filename);
+        report_printf(report, "\n%s: %s", summary, filename);
         free(summary); summary=NULL;
     } else {
-        if (node->checked&NODE_ADDED) {
-            report_printf(report, _("\nadded: %s"),(node->new_data)->filename);
-        } else if (node->checked&NODE_REMOVED) {
-            report_printf(report, _("\nremoved: %s"),(node->old_data)->filename);
-        } else if (node->checked&NODE_CHANGED) {
-            report_printf(report, _("\nchanged: %s"),(node->new_data)->filename);
+        if (node_checked&NODE_ADDED) {
+            report_printf(report, _("\nadded: %s"), filename);
+        } else if (node_checked&NODE_REMOVED) {
+            report_printf(report, _("\nremoved: %s"), filename);
+        } else if (node_checked&NODE_CHANGED) {
+            report_printf(report, _("\nchanged: %s"), filename);
         }
     }
 }
 
 static void print_report_dbline_attributes_plain(report_t *report, db_line* oline, db_line* nline, DB_ATTR_TYPE report_attrs) {
     if  (report_attrs)  {
-        char *file_type = get_file_type_string((nline==NULL?oline:nline)->perm);
+        db_line* line = nline==NULL?oline:nline;
         report_printf(report, "\n");
-        if (file_type) {
-            report_printf(report, "%s: ", file_type);
+        if (line->perm) {
+            report_printf(report, "%s: ", get_file_type_string(line->perm));
         }
-        report_printf(report, "%s\n", (nline==NULL?oline:nline)->filename);
+        report_printf(report, "%s\n", line->filename);
 
         print_dbline_attrs(report, oline, nline, report_attrs, _print_attribute);
     }
