@@ -1,7 +1,7 @@
 /*
  * AIDE (Advanced Intrusion Detection Environment)
  *
- * Copyright (C) 1999-2002, 2004-2006, 2010-2011, 2013, 2016, 2019-2022
+ * Copyright (C) 1999-2002, 2004-2006, 2010-2011, 2013, 2016, 2019-2023
  *               Rami Lehti, Pablo Virolainen, Mike Markley,
  *               Richard van den Berg, Hannes von Haugwitz
  *
@@ -28,8 +28,11 @@
 #include <ctype.h>
 #include <syslog.h>
 #include <stdbool.h>
+#include <pthread.h>
 #include <strings.h>
 #include <unistd.h>
+#include <signal.h>
+#include <time.h>
 #include "url.h"
 /*for locale support*/
 #include "locale-aide.h"
@@ -48,6 +51,41 @@
 #define URL_UNSAFE " <>\"#%{}|\\^~[]`@:\033'"
 #define ISPRINT(c) (isascii(c) && isprint(c))
 
+pthread_mutex_t stderr_mutex = PTHREAD_MUTEX_INITIALIZER;
+bool stderr_erase_line = false;
+
+void stderr_msg(const char* format, ...) {
+    pthread_mutex_lock(&stderr_mutex);
+    if (stderr_erase_line) {
+        fprintf(stderr, "\33[2K");
+    }
+    va_list ap;
+    va_start(ap, format);
+    vfprintf(stderr, format, ap);
+    va_end(ap);
+    pthread_mutex_unlock(&stderr_mutex);
+}
+
+void vstderr_prefix_line(const char* prefix, const char* format, va_list ap) {
+    pthread_mutex_lock(&stderr_mutex);
+    if (stderr_erase_line) {
+        fprintf(stderr, "\33[2K");
+    }
+    fprintf(stderr, "%s: ", prefix);
+    vfprintf(stderr, format, ap);
+    fprintf(stderr, "\n");
+    pthread_mutex_unlock(&stderr_mutex);
+}
+
+void stderr_set_line_erasure(bool erase_line) {
+    pthread_mutex_lock(&stderr_mutex);
+    if (stderr_erase_line) {
+        fprintf(stderr, "\33[2K");
+    }
+    stderr_erase_line = erase_line;
+    pthread_mutex_unlock(&stderr_mutex);
+}
+
 const char* btoa(bool b) {
     return b?"true":"false";
 }
@@ -55,7 +93,7 @@ const char* btoa(bool b) {
 void* checked_malloc(size_t size) {
     void * p = malloc(size);
     if (p == NULL) {
-        log_msg(LOG_LEVEL_ERROR, "malloc: failed to allocate %d bytes of memory", size);
+        log_msg(LOG_LEVEL_ERROR, "malloc: failed to allocate %lu bytes of memory", (unsigned long) size);
         exit(MEMORY_ALLOCATION_FAILURE);
     }
     return p;
@@ -63,7 +101,7 @@ void* checked_malloc(size_t size) {
 void* checked_calloc(size_t nmemb, size_t size) {
     void * p = calloc(nmemb, size);
     if (p == NULL) {
-        log_msg(LOG_LEVEL_ERROR, "calloc: failed to allocate %d bytes of memory", size);
+        log_msg(LOG_LEVEL_ERROR, "calloc: failed to allocate %lu bytes of memory", (unsigned long) size);
         exit(MEMORY_ALLOCATION_FAILURE);
     }
     return p;
@@ -79,7 +117,7 @@ void* checked_strdup(const char *s) {
 void* checked_strndup(const char *s, size_t size) {
     void * p = strndup(s, size);
     if (p == NULL) {
-        log_msg(LOG_LEVEL_ERROR, "strdup: failed to allocate memory");
+        log_msg(LOG_LEVEL_ERROR, "strndup: failed to allocate memory");
         exit(MEMORY_ALLOCATION_FAILURE);
     }
     return p;
@@ -87,7 +125,7 @@ void* checked_strndup(const char *s, size_t size) {
 void* checked_realloc(void *ptr, size_t size) {
     void * p = realloc(ptr,size);
     if (p == NULL) {
-        log_msg(LOG_LEVEL_ERROR, "realloc: failed to allocate memory");
+        log_msg(LOG_LEVEL_ERROR, "realloc: failed to re-allocate %lu bytes of memory", (unsigned long) size);
         exit(MEMORY_ALLOCATION_FAILURE);
     }
     return p;
@@ -178,6 +216,56 @@ char* encode_string (const char* s)
   return res;
 }
 
+char* byte_to_base16(const byte* src, size_t ssize) {
+    char* str = checked_malloc((2*ssize+1) * sizeof (char));
+    for(size_t i=0; i < ssize; ++i) {
+        snprintf(&str[2*i], 3, "%02x", src[i]);
+    }
+    return str;
+}
+
+char *get_progress_bar_string(const char* state_str, const char* path, long unsigned num_entries, long unsigned num_skipped, int elapsed, int length) {
+    char *progress_bar = checked_malloc(length+1);
+    int n = 0;
+    int left = length;
+    n += snprintf(&progress_bar[n], left+1, "[%02d:%02d] %s> %lu %s", elapsed/60, elapsed%60, state_str, num_entries, num_entries == 1?"file":"files");
+    left = length-n;
+    if (num_skipped && left > 0) {
+        n += snprintf(&progress_bar[n], left+1, " (%lu skipped)", num_skipped);
+        left = length-n;
+    }
+    if (path && left > 0) {
+        const char *ellipsis = "/...";
+        const char *last_str = ", last ";
+        int last_str_len = strlen(last_str);
+        int ellipsis_len = 0;
+        int prefix_len = 0;
+
+        const char *suffix_path = path;
+        if ((long) strlen(path) > (left-last_str_len) ) {
+            const char *first_slash = strchr(path+1, '/');
+            if (first_slash) {
+                ellipsis_len = strlen(ellipsis);
+                prefix_len = first_slash - path;
+
+                suffix_path = first_slash+1;
+
+                int path_left = left - last_str_len - prefix_len - ellipsis_len;
+                while ((long) strlen(suffix_path) > path_left) {
+                    char *slash = strchr(suffix_path+1, '/');
+                    if (slash) {
+                        suffix_path = slash;
+                    } else {
+                        break;
+                    }
+                }
+            }
+        }
+        snprintf(&progress_bar[n], left+1, "%s%.*s%.*s%s", last_str, prefix_len, path, ellipsis_len, ellipsis, suffix_path);
+    }
+    return progress_bar;
+}
+
 char* perm_to_char(mode_t perm)
 {
   char*pc=NULL;
@@ -309,6 +397,32 @@ char* pipe2string(int fd) {
     return str;
 }
 
+char* get_time_string(const time_t *tm) {
+    const char time_format[] = "%Y-%m-%d %H:%M:%S %z";
+    int time_string_len = strlen("1979-01-01 01:00:00 +0100")+1;
+
+    char *time = checked_malloc(time_string_len * sizeof (char));
+    strftime(time, time_string_len, time_format, localtime(tm));
+
+    return time;
+}
+
+void mask_sig(const char* whoami) {
+    sigset_t set;
+
+    sigemptyset(&set);
+    sigaddset(&set, SIGINT);
+    sigaddset(&set, SIGTERM);
+    sigaddset(&set, SIGHUP);
+    sigaddset(&set, SIGUSR1);
+    sigaddset(&set, SIGWINCH);
+
+    if(pthread_sigmask(SIG_BLOCK, &set, NULL)) {
+        log_msg(LOG_LEVEL_ERROR, "%10s: pthread_sigmask failed to set mask of blocked signals", whoami);
+        exit(THREAD_ERROR);
+    }
+}
+
 /* Like strstr but only do search for maximum of n chars.
    haystack does not have to be NULL terminated
    needle has to be NULL terminated. NULL in needle is not used in compare.
